@@ -58,7 +58,9 @@ int RUNMAIN(arcsub)(int argc, char **argv) {
 
   // If debug is specified as argument, it should be set before loading the configuration.
   if (!opt.debug.empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(opt.debug));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(opt.debug));
+
+  logger.msg(Arc::VERBOSE, "Running command: %s", opt.GetCommandWithArguments());
 
   Arc::UserConfig usercfg(opt.conffile, opt.joblist);
   if (!usercfg) {
@@ -82,7 +84,7 @@ int RUNMAIN(arcsub)(int argc, char **argv) {
   }
 
   if (opt.debug.empty() && !usercfg.Verbosity().empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(usercfg.Verbosity()));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(usercfg.Verbosity()));
 
   if (opt.timeout > 0)
     usercfg.Timeout(opt.timeout);
