@@ -79,8 +79,9 @@ int RUNMAIN(arctest)(int argc, char **argv) {
 
   // If debug is specified as argument, it should be set before loading the configuration.
   if (!opt.debug.empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(opt.debug));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(opt.debug));
 
+  logger.msg(Arc::VERBOSE, "Running command: %s", opt.GetCommandWithArguments());
 
   Arc::UserConfig usercfg(opt.conffile, opt.joblist);
   if (!usercfg) {
@@ -100,7 +101,7 @@ int RUNMAIN(arctest)(int argc, char **argv) {
   }
 
   if (opt.debug.empty() && !usercfg.Verbosity().empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(usercfg.Verbosity()));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(usercfg.Verbosity()));
 
   if (opt.show_credentials) {
     const Arc::Time now;
@@ -253,6 +254,12 @@ int RUNMAIN(arctest)(int argc, char **argv) {
   if (opt.dumpdescription) {
      return dumpjobdescription(usercfg, ets, testJob);
   }
+  
+  std::cout << Arc::IString("Submitting test-job %d:", opt.testjobid) << std::endl;
+  std::string testJobXRSL;
+  testJob.UnParse(testJobXRSL, "nordugrid:xrsl");
+  std::cout << testJobXRSL << std::endl;
+  std::cout << Arc::IString("Client version: nordugrid-arc-%s", VERSION) << std::endl;
   return test(usercfg, ets, testJob, opt.jobidoutfile);
 }
 
@@ -275,6 +282,7 @@ int test(const Arc::UserConfig& usercfg, Arc::ExecutionTargetSorter& ets, const
   for (ets.reset(); !ets.endOfList(); ets.next()) {
     if (ets->Submit(usercfg, testJob, submittedJobs.back())) {
       printjobid(submittedJobs.back().JobID, jobidfile);
+      std::cout << Arc::IString("Computing service: %s", ets->ComputingService->Name) << std::endl;
       break;
     }
   }
