@@ -4,40 +4,22 @@
 #include <config.h>
 #endif
 
-#include <iostream>
-#include <fstream>
-#include <string>
-#include <map>
-#include <stdexcept>
 #include <unistd.h>
-#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <glibmm/stringutils.h>
-#include <glibmm/fileutils.h>
-#include <glibmm.h>
-#include <unistd.h>
 
 #include <arc/ArcLocation.h>
-#include <arc/Logger.h>
-#include <arc/DateTime.h>
-#include <arc/delegation/DelegationInterface.h>
 #include <arc/OptionParser.h>
 #include <arc/StringConv.h>
-#include <arc/User.h>
 #include <arc/Utils.h>
 #include <arc/UserConfig.h>
+#include <arc/FileUtils.h>
 #include <arc/communication/ClientInterface.h>
-#include <arc/credential/VOMSAttribute.h>
 #include <arc/credential/VOMSUtil.h>
 #include <arc/credential/Credential.h>
-#include <arc/credential/CertUtil.h>
 #include <arc/credentialstore/CredentialStore.h>
 #include <arc/crypto/OpenSSL.h>
-#include <arc/FileUtils.h>
-
-#include <openssl/ui.h>
 
 #ifdef HAVE_NSS
 #include <arc/credential/NSSUtil.h>
@@ -64,63 +46,28 @@ static void create_proxy(std::string& proxy_cert, Arc::Credential& signer,
 
 static std::string get_proxypolicy(const std::string& policy_source);
 
-static int create_proxy_file(const std::string& path) {
-  int f = -1;
-
-  if((::unlink(path.c_str()) != 0) && (errno != ENOENT)) {
-    throw std::runtime_error("Failed to remove proxy file " + path);
-  }
-  f = ::open(path.c_str(), O_WRONLY | O_CREAT | O_EXCL | O_TRUNC, S_IRUSR | S_IWUSR);
-  if (f == -1) {
-    throw std::runtime_error("Failed to create proxy file " + path);
-  }
-  if(::chmod(path.c_str(), S_IRUSR | S_IWUSR) != 0) {
-    ::unlink(path.c_str());
-    ::close(f);
-    throw std::runtime_error("Failed to change permissions of proxy file " + path);
-  }
-  return f;
-}
-
 static void write_proxy_file(const std::string& path, const std::string& content) {
   std::string::size_type off = 0;
-  int f = create_proxy_file(path);
-  while(off < content.length()) {
-    ssize_t l = ::write(f, content.c_str(), content.length()-off);
-    if(l < 0) {
-      ::unlink(path.c_str());
-      ::close(f);
-      throw std::runtime_error("Failed to write into proxy file " + path);
-    }
-    off += (std::string::size_type)l;
+  if((!Arc::FileDelete(path)) && (errno != ENOENT)) {
+    throw std::runtime_error("Failed to remove proxy file " + path);
+  }
+  if(!Arc::FileCreate(path, content, 0, 0, S_IRUSR | S_IWUSR)) {
+    throw std::runtime_error("Failed to create proxy file " + path);
   }
-  ::close(f);
 }
 
 static void remove_proxy_file(const std::string& path) {
-  if((::unlink(path.c_str()) != 0) && (errno != ENOENT)) {
+  if((!Arc::FileDelete(path)) && (errno != ENOENT)) {
     throw std::runtime_error("Failed to remove proxy file " + path);
   }
 }
 
 static void remove_cert_file(const std::string& path) {
-  if((::unlink(path.c_str()) != 0) && (errno != ENOENT)) {
+  if((!Arc::FileDelete(path)) && (errno != ENOENT)) {
     throw std::runtime_error("Failed to remove certificate file " + path);
   }
 }
 
-static void tls_process_error(Arc::Logger& logger) {
-  unsigned long err;
-  err = ERR_get_error();
-  if (err != 0) {
-    logger.msg(Arc::ERROR, "OpenSSL error -- %s", ERR_error_string(err, NULL));
-    logger.msg(Arc::ERROR, "Library  : %s", ERR_lib_error_string(err));
-    logger.msg(Arc::ERROR, "Function : %s", ERR_func_error_string(err));
-    logger.msg(Arc::ERROR, "Reason   : %s", ERR_reason_error_string(err));
-  }
-  return;
-}
-
 static bool is_file(std::string path) {
   if (Glib::file_test(path, Glib::FILE_TEST_IS_REGULAR))
     return true;
@@ -389,7 +336,8 @@ typedef enum {
   pass_all,
   pass_private_key,
   pass_myproxy,
-  pass_myproxy_new
+  pass_myproxy_new,
+  pass_nss
 } pass_destination_type;
 
 std::map<pass_destination_type, Arc::PasswordSource*> passsources;
@@ -464,7 +412,21 @@ int main(int argc, char *argv[]) {
                                     "  keybits - size of proxy certificate key in bits.\n"
                                     "  signingAlgorithm - algorith used to sign proxy certificate.\n"
                                     "Items are printed in requested order and are separated by newline.\n"
-                                    "If item has multiple values they are printed in same line separated by |."
+                                    "If item has multiple values they are printed in same line separated by |.\n"
+                                    "\n"
+                                    "Supported password destinations are:\n"
+                                    "  key - for reading private key\n"
+                                    "  myproxy - for accessing credentials at MyProxy service\n"
+                                    "  myproxynew - for creating credentials at MyProxy service\n"
+                                    "  all - for any purspose.\n"
+                                    "\n"
+                                    "Supported password sources are:\n"
+                                    "  quoted string (\"password\") - explicitely specified password\n"
+                                    "  int - interactively request password from console\n"
+                                    "  stdin - read password from standard input delimited by newline\n"
+                                    "  file:filename - read password from file named filename\n"
+                                    "  stream:# - read password from input stream number #.\n"
+                                    "             Currently only 0 (standard input) is supported.\n"
   ));
 
   std::string proxy_path;
@@ -569,8 +531,8 @@ int main(int argc, char *argv[]) {
                                                ),
                     istring("string"), myproxy_command);
 
-#ifdef HAVE_NSS
   bool use_nssdb = false;
+#ifdef HAVE_NSS
   options.AddOption('F', "nssdb", istring("use NSS credential database in default Mozilla profiles, \n"
                                           "              including Firefox, Seamonkey and Thunderbird.\n"), use_nssdb);
 #endif
@@ -580,7 +542,7 @@ int main(int argc, char *argv[]) {
                     istring("string"), constraintlist);
 
   std::list<std::string> passsourcelist;
-  options.AddOption('p', "passwordsource", istring("password sources"),
+  options.AddOption('p', "passwordsource", istring("password destination=password source"),
                     istring("string"), passsourcelist);
 
   int timeout = -1;
@@ -623,8 +585,7 @@ int main(int argc, char *argv[]) {
   // Set default, predefined or guessed credentials. Also check if they exist.
 #ifdef HAVE_NSS
   Arc::UserConfig usercfg(conffile,
-        Arc::initializeCredentialsType(use_nssdb ? Arc::initializeCredentialsType::NotTryCredentials 
-        : Arc::initializeCredentialsType::TryCredentials));
+        Arc::initializeCredentialsType(Arc::initializeCredentialsType::TryCredentials));
 #else
   Arc::UserConfig usercfg(conffile,
         Arc::initializeCredentialsType(Arc::initializeCredentialsType::TryCredentials));
@@ -633,11 +594,15 @@ int main(int argc, char *argv[]) {
     logger.msg(Arc::ERROR, "Failed configuration initialization.");
     return EXIT_FAILURE;
   }
+  if(use_nssdb) {
+    usercfg.CertificatePath("");;
+    usercfg.KeyPath("");;
+  }
 
   // Check for needed credentials objects
   // Can proxy be used for? Could not find it in documentation.
   // Key and certificate not needed if only printing proxy information
-  if (!(Arc::lower(myproxy_command) == "get")) {
+  if ( (!(Arc::lower(myproxy_command) == "get")) && (!use_nssdb) ) {
     if((usercfg.CertificatePath().empty() || 
         (
          usercfg.KeyPath().empty() && 
@@ -699,13 +664,14 @@ int main(int argc, char *argv[]) {
                  "please setup environment X509_USER_PROXY, "
                  "or proxypath in a configuration file");
       return EXIT_FAILURE;
-    } else if (!(Glib::file_test(proxy_path, Glib::FILE_TEST_EXISTS))) {
-        logger.msg(Arc::ERROR, "Cannot remove proxy file at %s, because it's not there", proxy_path);
-        return EXIT_FAILURE;
     }
-    if((unlink(proxy_path.c_str()) != 0) && (errno != ENOENT)) {
+    if(!Arc::FileDelete(proxy_path)) {
+      if(errno != ENOENT) {
         logger.msg(Arc::ERROR, "Cannot remove proxy file at %s", proxy_path);
-        return EXIT_FAILURE;
+      } else {
+        logger.msg(Arc::ERROR, "Cannot remove proxy file at %s, because it's not there", proxy_path);
+      }
+      return EXIT_FAILURE;
     }
     return EXIT_SUCCESS;
   }
@@ -957,18 +923,27 @@ int main(int argc, char *argv[]) {
     pass_destination_type pass_dest;
     if(dest == "key") {
       pass_dest = pass_private_key;
+    } else if(dest == "myproxy") {
+      pass_dest = pass_myproxy;
+    } else if(dest == "myproxynew") {
+      pass_dest = pass_myproxy_new;
+    } else if(dest == "nss") {
+      pass_dest = pass_nss;
+    } else if(dest == "all") {
+      pass_dest = pass_all;
     } else {
       logger.msg(Arc::ERROR, "Cannot parse password type %s. "
-                 "Currently supported value is 'key'.", dest);
+                 "Currently supported values are 'key','myproxy','myproxynew' and 'all'.", dest);
       return EXIT_FAILURE;
     }
+    Arc::PasswordSource* pass_source;
     std::string pass = it->substr(pos + 1);
     if((pass[0] == '"') && (pass[pass.length()-1] == '"')) {
-      passsources[pass_dest] = new Arc::PasswordSourceString(pass.substr(1,pass.length()-2));
+      pass_source = new Arc::PasswordSourceString(pass.substr(1,pass.length()-2));
     } else if(pass == "int") {
-      passsources[pass_dest] = new Arc::PasswordSourceInteractive(passprompts[pass_private_key].first,passprompts[pass_private_key].second);
+      pass_source = new Arc::PasswordSourceInteractive(passprompts[pass_private_key].first,passprompts[pass_private_key].second);
     } else if(pass == "stdin") {
-      passsources[pass_dest] = new Arc::PasswordSourceStream(&std::cin);
+      pass_source = new Arc::PasswordSourceStream(&std::cin);
     } else {
       pos = pass.find(':');
       if(pos == std::string::npos) {
@@ -980,11 +955,11 @@ int main(int argc, char *argv[]) {
       std::string data = pass.substr(pos + 1);
       pass.resize(pos);
       if(pass == "file") {
-        passsources[pass_dest] = new PasswordSourceFile(data);
+        pass_source = new PasswordSourceFile(data);
         // TODO: combine same files
       } else if(pass == "stream") {
         if(data == "0") {
-          passsources[pass_dest] = new Arc::PasswordSourceStream(&std::cin);
+          pass_source = new Arc::PasswordSourceStream(&std::cin);
         } else {
           logger.msg(Arc::ERROR, "Only standard input is currently supported "
                      "for password source.");
@@ -996,6 +971,16 @@ int main(int argc, char *argv[]) {
         return EXIT_FAILURE;
       }
     }
+    if(pass_source) {
+      if(pass_dest != pass_all) {
+        passsources[pass_dest] = pass_source;
+      } else {
+        passsources[pass_private_key] = pass_source;
+        passsources[pass_myproxy] = pass_source;
+        passsources[pass_myproxy_new] = pass_source;
+        passsources[pass_nss] = pass_source;
+      }
+    }
   }
   for(std::map<pass_destination_type, std::pair<std::string,bool> >::iterator p = passprompts.begin();
                             p != passprompts.end();++p) {
@@ -1011,7 +996,7 @@ int main(int argc, char *argv[]) {
   // TODO: Is default validityPeriod since now or since validityStart?
   Arc::Time validityStart = now; // now by default
   Arc::Period validityPeriod(12*60*60);
-  if (myproxy_command == "put" || myproxy_command == "PUT" || myproxy_command == "Put") {
+  if (Arc::lower(myproxy_command) == "put") {
     //For myproxy PUT operation, the proxy should be 7 days according to the default 
     //definition in myproxy implementation.
     validityPeriod = 7*24*60*60;
@@ -1127,29 +1112,29 @@ int main(int argc, char *argv[]) {
     // if multiple profiles exist
     bool res;
     std::string configdir;
-    if(nssdb_paths.size()) {
-      std::cout<<Arc::IString("There are %d NSS base directories where the certificate, key, and module datbases live",
-        nssdb_paths.size())<<std::endl;
-    }
-    for(int i=0; i < nssdb_paths.size(); i++) {
-      std::cout<<Arc::IString("Number %d is: %s", i+1, nssdb_paths[i])<<std::endl;
-    }
-    std::cout << Arc::IString("Please choose the NSS database you would use (1-%d): ", nssdb_paths.size());
-    if(nssdb_paths.size() == 1) { configdir = nssdb_paths[0]; }
-    char c;
-    while(true && (nssdb_paths.size()>1)) {
-      c = getchar();
-      int num = c - '0';
-      if((num<=nssdb_paths.size()) && (num>=1)) {
-        configdir = nssdb_paths[num-1];
-        break;
+    if(nssdb_paths.size() > 1) {
+      std::cout<<Arc::IString("There are %d NSS base directories where the certificate, key, and module databases live", nssdb_paths.size())<<std::endl;
+      for(int i=0; i < nssdb_paths.size(); i++) {
+        std::cout<<Arc::IString("Number %d is: %s", i+1, nssdb_paths[i])<<std::endl;
       }
+      std::cout << Arc::IString("Please choose the NSS database you would like to use (1-%d): ", nssdb_paths.size());
+      // todo: more than 9 paths
+      while(true) {
+        char c;
+        c = getchar();
+        int num = c - '0';
+        if((num<=nssdb_paths.size()) && (num>=1)) {
+          configdir = nssdb_paths[num-1];
+          break;
+        }
+      }
+    } else {
+      configdir = nssdb_paths[0];
     }
 
     res = AuthN::nssInit(configdir);
     std::cout<< Arc::IString("NSS database to be accessed: %s\n", configdir.c_str());
 
-    char* slotpw = NULL; //"secretpw";  
     //The nss db under firefox profile seems to not be protected by any passphrase by default
     bool ascii = true;
     const char* trusts = "u,u,u";
@@ -1158,7 +1143,7 @@ int main(int argc, char *argv[]) {
     std::string proxy_csrfile = "proxy.csr";
     std::string proxy_keyname = "proxykey";
     std::string proxy_privk_str;
-    res = AuthN::nssGenerateCSR(proxy_keyname, "CN=Test,OU=ARC,O=EMI", slotpw, proxy_csrfile, proxy_privk_str, ascii);
+    res = AuthN::nssGenerateCSR(proxy_keyname, "CN=Test,OU=ARC,O=EMI", *passsources[pass_nss], proxy_csrfile, proxy_privk_str, ascii);
     if(!res) return EXIT_FAILURE;
 
     // Create a temporary proxy and contact voms server
@@ -1166,22 +1151,35 @@ int main(int argc, char *argv[]) {
     std::string vomsacseq;
     if (!vomslist.empty()) {
       std::string tmp_proxy_path;
-      tmp_proxy_path = Glib::build_filename(Glib::get_tmp_dir(), std::string("tmp_proxy.pem"));
+      if(!Arc::TmpFileCreate(tmp_proxy_path,"")) return EXIT_FAILURE;
+
       get_nss_certname(issuername, logger);
       
       // Create tmp proxy cert
       int duration = 12;
       res = AuthN::nssCreateCert(proxy_csrfile, issuername, NULL, duration, "", tmp_proxy_path, ascii);
-      if(!res) return EXIT_FAILURE;
+      if(!res) {
+        remove_proxy_file(tmp_proxy_path);
+        return EXIT_FAILURE;
+      }
+      // TODO: Use FileUtils
       std::string tmp_proxy_cred_str;
       std::ifstream tmp_proxy_cert_s(tmp_proxy_path.c_str());
       std::getline(tmp_proxy_cert_s, tmp_proxy_cred_str,'\0');
       tmp_proxy_cert_s.close();
 
       // Export EEC
-      std::string cert_file = "cert.pem";
+      std::string cert_file;
+      if(!Arc::TmpFileCreate(cert_file,"")) {
+        remove_proxy_file(tmp_proxy_path);
+        return EXIT_FAILURE;
+      }
       res = AuthN::nssExportCertificate(issuername, cert_file);
-      if(!res) return EXIT_FAILURE;
+      if(!res) {
+        remove_cert_file(cert_file);
+        remove_proxy_file(tmp_proxy_path);
+        return EXIT_FAILURE;
+      }
       std::string eec_cert_str;
       std::ifstream eec_s(cert_file.c_str());
       std::getline(eec_s, eec_cert_str,'\0');
@@ -1213,7 +1211,7 @@ int main(int argc, char *argv[]) {
     if(!res) return EXIT_FAILURE;
 
     const char* proxy_certname = "proxycert";
-    res = AuthN::nssImportCert(slotpw, proxy_certfile, proxy_certname, trusts, ascii);
+    res = AuthN::nssImportCert(*passsources[pass_nss], proxy_certfile, proxy_certname, trusts, ascii);
     if(!res) return EXIT_FAILURE;
 
     //Compose the proxy certificate 
@@ -1333,7 +1331,10 @@ int main(int argc, char *argv[]) {
       //Generate a temporary self-signed proxy certificate
       //to contact the voms server
       std::string tmp_proxy_path;
-      tmp_proxy_path = Glib::build_filename(Glib::get_tmp_dir(), std::string("tmp_proxy.pem"));
+      if(!Arc::TmpFileCreate(tmp_proxy_path,"")) {
+        std::cerr << Arc::IString("Proxy generation failed: Failed to create temporary file.") << std::endl;
+        return EXIT_FAILURE;
+      }
       create_tmp_proxy(tmp_proxy_path, signer);
       contact_voms_servers(vomslist, orderlist, vomses_path, use_gsi_comm,
           use_http_comm, voms_period, usercfg, logger, tmp_proxy_path, vomsacseq);
@@ -1358,7 +1359,6 @@ int main(int argc, char *argv[]) {
     //return EXIT_SUCCESS;
   } catch (std::exception& err) {
     logger.msg(Arc::ERROR, err.what());
-    tls_process_error(logger);
     return EXIT_FAILURE;
   }
 
@@ -1672,17 +1672,10 @@ static std::string get_proxypolicy(const std::string& policy_source) {
     //If the argument is a location which specifies a file that
     //includes the policy content
     if(Glib::file_test(policy_source, Glib::FILE_TEST_IS_REGULAR)) {
-      std::ifstream fp;
-      fp.open(policy_source.c_str());
-      if(!fp) {
-        std::cout << Arc::IString("Error: can't open policy file: %s", policy_source.c_str()) << std::endl;
+      if(!Arc::FileRead(policy_source, policystring)) {
+        std::cout << Arc::IString("Error: can't read policy file: %s", policy_source.c_str()) << std::endl;
         return std::string();
       }
-      fp.unsetf(std::ios::skipws);
-      char c;
-      while(fp.get(c))
-        policystring += c;
-      fp.close();
     }
     else {
       std::cout << Arc::IString("Error: policy location: %s is not a regular file", policy_source.c_str()) <<std::endl;
@@ -1753,7 +1746,6 @@ static bool contact_myproxy_server(const std::string& myproxy_server, const std:
 
   } catch (std::exception& err) {
     logger.msg(Arc::ERROR, err.what());
-    tls_process_error(logger);
     return false;
   }
 
@@ -1801,7 +1793,6 @@ static bool contact_myproxy_server(const std::string& myproxy_server, const std:
 
   } catch (std::exception& err) {
     logger.msg(Arc::ERROR, err.what());
-    tls_process_error(logger);
     return false;
   }
 
@@ -1845,7 +1836,6 @@ static bool contact_myproxy_server(const std::string& myproxy_server, const std:
     }
   } catch (std::exception& err) {
     logger.msg(Arc::ERROR, err.what());
-    tls_process_error(logger);
     return false;
   }
 
@@ -1930,7 +1920,6 @@ static bool contact_myproxy_server(const std::string& myproxy_server, const std:
 
   } catch (std::exception& err) {
     logger.msg(Arc::ERROR, err.what());
-    tls_process_error(logger);
     return false;
   }
 
@@ -1984,7 +1973,6 @@ static bool contact_myproxy_server(const std::string& myproxy_server, const std:
     }
   } catch (std::exception& err) {
     logger.msg(Arc::ERROR, err.what());
-    tls_process_error(logger);
     remove_proxy_file(proxy_path);
     return false;
   }
