@@ -349,7 +349,9 @@ int main(int argc, char **argv) {
 
   // If debug is specified as argument, it should be set before loading the configuration.
   if (!debug.empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(debug));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(debug));
+
+  logger.msg(Arc::VERBOSE, "Running command: %s", options.GetCommandWithArguments());
 
   if (show_plugins) {
     std::list<Arc::ModuleDesc> modules;
@@ -377,7 +379,7 @@ int main(int argc, char **argv) {
   usercfg.UtilsDirPath(Arc::UserConfig::ARCUSERDIRECTORY);
 
   if (debug.empty() && !usercfg.Verbosity().empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(usercfg.Verbosity()));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(usercfg.Verbosity()));
 
   // Analyze options
 
