@@ -65,7 +65,7 @@ int main(int argc, char **argv) {
 
   // If debug is specified as argument, it should be set before loading the configuration.
   if (!debug.empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(debug));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(debug));
 
   Arc::UserConfig usercfg(conffile);
   if (!usercfg) {
@@ -74,7 +74,7 @@ int main(int argc, char **argv) {
   }
 
   if (debug.empty() && !usercfg.Verbosity().empty())
-    Arc::Logger::getRootLogger().setThreshold(Arc::string_to_level(usercfg.Verbosity()));
+    Arc::Logger::getRootLogger().setThreshold(Arc::istring_to_level(usercfg.Verbosity()));
 
   if (timeout > 0) {
     usercfg.Timeout(timeout);
