@@ -9,7 +9,7 @@
 #include <arc/StringConv.h>
 #include <arc/UserConfig.h>
 #include <arc/XMLNode.h>
-#include <arc/client/JobDescription.h>
+#include <arc/compute/JobDescription.h>
 #include <arc/data/DataMover.h>
 #include <arc/data/DataHandle.h>
 #include <arc/data/URLMap.h>
@@ -25,7 +25,7 @@ namespace Arc {
   Logger JobControllerPluginARC1::logger(Logger::getRootLogger(), "JobControllerPlugin.ARC1");
   
   URL JobControllerPluginARC1::GetAddressOfResource(const Job& job) {
-    return URL(XMLNode(job.IDFromEndpoint)["Address"]); 
+    return job.ServiceInformationURL; 
   }
 
   bool JobControllerPluginARC1::isEndpointNotSupported(const std::string& endpoint) const {
@@ -33,13 +33,13 @@ namespace Arc {
     return pos != std::string::npos && lower(endpoint.substr(0, pos)) != "http" && lower(endpoint.substr(0, pos)) != "https";
   }
 
-  void JobControllerPluginARC1::UpdateJobs(std::list<Job*>& jobs, std::list<URL>& IDsProcessed, std::list<URL>& IDsNotProcessed, bool isGrouped) const {
+  void JobControllerPluginARC1::UpdateJobs(std::list<Job*>& jobs, std::list<std::string>& IDsProcessed, std::list<std::string>& IDsNotProcessed, bool isGrouped) const {
     for (std::list<Job*>::iterator it = jobs.begin(); it != jobs.end(); ++it) {
       AutoPointer<AREXClient> ac(((AREXClients&)clients).acquire(GetAddressOfResource(**it), true));
       std::string idstr;
-      AREXClient::createActivityIdentifier((*it)->JobID, idstr);
+      AREXClient::createActivityIdentifier(URL((*it)->JobID), idstr);
       if (!ac->stat(idstr, **it)) {
-        logger.msg(WARNING, "Job information not found in the information system: %s", (*it)->JobID.fullstr());
+        logger.msg(WARNING, "Job information not found in the information system: %s", (*it)->JobID);
         IDsNotProcessed.push_back((*it)->JobID);
         ((AREXClients&)clients).release(ac.Release());
         continue;
@@ -49,13 +49,13 @@ namespace Arc {
     }
   }
 
-  bool JobControllerPluginARC1::CleanJobs(const std::list<Job*>& jobs, std::list<URL>& IDsProcessed, std::list<URL>& IDsNotProcessed, bool isGrouped) const {
+  bool JobControllerPluginARC1::CleanJobs(const std::list<Job*>& jobs, std::list<std::string>& IDsProcessed, std::list<std::string>& IDsNotProcessed, bool isGrouped) const {
     bool ok = true;
     for (std::list<Job*>::const_iterator it = jobs.begin(); it != jobs.end(); ++it) {
       Job& job = **it;
       AutoPointer<AREXClient> ac(((AREXClients&)clients).acquire(GetAddressOfResource(job), true));
       std::string idstr;
-      AREXClient::createActivityIdentifier(job.JobID, idstr);
+      AREXClient::createActivityIdentifier(URL(job.JobID), idstr);
       if (!ac->clean(idstr)) {
         ok = false;
         IDsNotProcessed.push_back(job.JobID);
@@ -69,13 +69,13 @@ namespace Arc {
     return ok;
   }
 
-  bool JobControllerPluginARC1::CancelJobs(const std::list<Job*>& jobs, std::list<URL>& IDsProcessed, std::list<URL>& IDsNotProcessed, bool isGrouped) const {
+  bool JobControllerPluginARC1::CancelJobs(const std::list<Job*>& jobs, std::list<std::string>& IDsProcessed, std::list<std::string>& IDsNotProcessed, bool isGrouped) const {
     bool ok = true;
     for (std::list<Job*>::const_iterator it = jobs.begin(); it != jobs.end(); ++it) {
       Job& job = **it;
       AutoPointer<AREXClient> ac(((AREXClients&)clients).acquire(GetAddressOfResource(job), true));
       std::string idstr;
-      AREXClient::createActivityIdentifier(job.JobID, idstr);
+      AREXClient::createActivityIdentifier(URL(job.JobID), idstr);
       if (!ac->kill(idstr)) {
         ok = false;
         IDsNotProcessed.push_back(job.JobID);
@@ -90,7 +90,7 @@ namespace Arc {
     return ok;
   }
 
-  bool JobControllerPluginARC1::RenewJobs(const std::list<Job*>& jobs, std::list<URL>& IDsProcessed, std::list<URL>& IDsNotProcessed, bool isGrouped) const {
+  bool JobControllerPluginARC1::RenewJobs(const std::list<Job*>& jobs, std::list<std::string>& IDsProcessed, std::list<std::string>& IDsNotProcessed, bool isGrouped) const {
     for (std::list<Job*>::const_iterator it = jobs.begin(); it != jobs.end(); ++it) {
       logger.msg(INFO, "Renewal of ARC1 jobs is not supported");
       IDsNotProcessed.push_back((*it)->JobID);
@@ -98,22 +98,22 @@ namespace Arc {
     return false;
   }
 
-  bool JobControllerPluginARC1::ResumeJobs(const std::list<Job*>& jobs, std::list<URL>& IDsProcessed, std::list<URL>& IDsNotProcessed, bool isGrouped) const {
+  bool JobControllerPluginARC1::ResumeJobs(const std::list<Job*>& jobs, std::list<std::string>& IDsProcessed, std::list<std::string>& IDsNotProcessed, bool isGrouped) const {
     bool ok = true;
     for (std::list<Job*>::const_iterator it = jobs.begin(); it != jobs.end(); ++it) {
       Job& job = **it;
       if (!job.RestartState) {
-        logger.msg(INFO, "Job %s does not report a resumable state", job.JobID.fullstr());
+        logger.msg(INFO, "Job %s does not report a resumable state", job.JobID);
         ok = false;
         IDsNotProcessed.push_back(job.JobID);
         continue;
       }
   
-      logger.msg(VERBOSE, "Resuming job: %s at state: %s (%s)", job.JobID.fullstr(), job.RestartState.GetGeneralState(), job.RestartState());
+      logger.msg(VERBOSE, "Resuming job: %s at state: %s (%s)", job.JobID, job.RestartState.GetGeneralState(), job.RestartState());
   
       AutoPointer<AREXClient> ac(((AREXClients&)clients).acquire(GetAddressOfResource(job), true));
       std::string idstr;
-      AREXClient::createActivityIdentifier(job.JobID, idstr);
+      AREXClient::createActivityIdentifier(URL(job.JobID), idstr);
       if (!ac->resume(idstr)) {
         ok = false;
         IDsNotProcessed.push_back(job.JobID);
@@ -130,7 +130,11 @@ namespace Arc {
   }
 
   bool JobControllerPluginARC1::GetURLToJobResource(const Job& job, Job::ResourceType resource, URL& url) const {
-    url = job.JobID;
+    url = URL(job.JobID);
+    // compensate for time between request and response on slow networks
+    url.AddOption("threads=2",false);
+    url.AddOption("encryption=optional",false);
+    url.AddOption("httpputpartial=yes",false);
     switch (resource) {
     case Job::STDIN:
       url.ChangePath(url.Path() + '/' + job.StdIn);
@@ -148,7 +152,7 @@ namespace Arc {
     case Job::JOBLOG:
     case Job::JOBDESCRIPTION:
       std::string path = url.Path();
-      path.insert(path.rfind('/'), "/info");
+      path.insert(path.rfind('/'), "/?logs");
       url.ChangePath(path + (Job::JOBLOG ? "/errors" : "/description"));
       break;
     }
@@ -161,7 +165,7 @@ namespace Arc {
     usercfg.ApplyToConfig(cfg);
     AutoPointer<AREXClient> ac(((AREXClients&)clients).acquire(GetAddressOfResource(job), true));
     std::string idstr;
-    AREXClient::createActivityIdentifier(job.JobID, idstr);
+    AREXClient::createActivityIdentifier(URL(job.JobID), idstr);
     if (ac->getdesc(idstr, desc_str)) {
       std::list<JobDescription> descs;
       if (JobDescription::Parse(desc_str, descs) && !descs.empty()) {
@@ -171,7 +175,7 @@ namespace Arc {
     }
     ((AREXClients&)clients).release(ac.Release());
 
-    logger.msg(ERROR, "Failed retrieving job description for job: %s", job.JobID.fullstr());
+    logger.msg(ERROR, "Failed retrieving job description for job: %s", job.JobID);
     return false;
   }
 } // namespace Arc
