@@ -32,29 +32,8 @@ namespace Arc {
     return pos != std::string::npos && lower(endpoint.substr(0, pos)) != "http" && lower(endpoint.substr(0, pos)) != "https";
   }
 
-  static URL CreateURL(std::string str) {
-    std::string::size_type pos1 = str.find("://");
-    if (pos1 == std::string::npos) {
-      str = "https://" + str;
-      pos1 = 5;
-    } else {
-      if(lower(str.substr(0,pos1)) != "https" && lower(str.substr(0,pos1)) != "http" ) return URL();
-    }
-    std::string::size_type pos2 = str.find(":", pos1 + 3);
-    std::string::size_type pos3 = str.find("/", pos1 + 3);
-    if (pos3 == std::string::npos) {
-      if (pos2 == std::string::npos)
-        str += ":443";
-      str += "/arex";
-    }
-    else if (pos2 == std::string::npos || pos2 > pos3)
-      str.insert(pos3, ":443");
-
-    return str;
-  }
-
   SubmissionStatus SubmitterPluginARC1::Submit(const std::list<JobDescription>& jobdescs, const std::string& endpoint, EntityConsumer<Job>& jc, std::list<const JobDescription*>& notSubmitted) {
-    URL url = CreateURL(endpoint);
+    URL url((endpoint.find("://") == std::string::npos ? "https://" : "") + endpoint, false, 443, "/arex");
 
     // TODO: Determine extended BES interface interface (A-REX WS)
     bool arex_features = true; //et.ComputingService->Type == "org.nordugrid.execution.arex";
