@@ -31,22 +31,6 @@ namespace Arc {
     return pos != std::string::npos && lower(endpoint.substr(0, pos)) != "http" && lower(endpoint.substr(0, pos)) != "https";
   }
   
-  static URL CreateURL(std::string str) {
-    std::string::size_type pos1 = str.find("://");
-    if (pos1 == std::string::npos) {
-      str = "https://" + str;
-      pos1 = 5;
-    } else {
-      if(lower(str.substr(0,pos1)) != "https" && lower(str.substr(0,pos1)) != "http" ) return URL();
-    }
-    std::string::size_type pos2 = str.find(":", pos1 + 3);
-    std::string::size_type pos3 = str.find("/", pos1 + 3);
-    if (pos3 == std::string::npos && pos2 == std::string::npos)  str += ":443";
-    else if (pos2 == std::string::npos || pos2 > pos3) str.insert(pos3, ":443");
-
-    return str;
-  }
-  
   bool SubmitterPluginEMIES::getDelegationID(const URL& durl, std::string& delegation_id) {
     if(!durl) {
       logger.msg(INFO, "Failed to delegate credentials to server - no delegation interface found");
@@ -69,7 +53,7 @@ namespace Arc {
     // TODO: this is multi step process. So having retries would be nice.
     // TODO: If delegation interface is not on same endpoint as submission interface this method is faulty.
 
-    URL url = CreateURL(endpoint);
+    URL url((endpoint.find("://") == std::string::npos ? "https://" : "") + endpoint, false, 443);
 
     SubmissionStatus retval;
     bool need_delegation = false;
@@ -118,15 +102,14 @@ namespace Arc {
       }
 
       if (need_delegation && delegation_id.empty()) {
-      // Assume that delegation interface is on same machine as submission interface.
-      if (need_delegation && delegation_id.empty() && !getDelegationID(url, delegation_id)) {
-        notSubmitted.push_back(&*itJ);
-        retval |= SubmissionStatus::DESCRIPTION_NOT_SUBMITTED;
-        products.pop_back();
-        have_uploads.pop_back();
-        continue;
-      }
-      
+        // Assume that delegation interface is on same machine as submission interface.
+        if (!getDelegationID(url, delegation_id)) {
+          notSubmitted.push_back(&*itJ);
+          retval |= SubmissionStatus::DESCRIPTION_NOT_SUBMITTED;
+          products.pop_back();
+          have_uploads.pop_back();
+          continue;
+        }
       }
 
       if(have_uploads.back()) {
@@ -166,6 +149,10 @@ namespace Arc {
         if(!j->manager) {
           j->manager = url;
         }
+
+        if(j->delegation_id.empty()) {
+          j->delegation_id = delegation_id;
+        }
   
         JobDescription preparedjobdesc(*itJ);
         preparedjobdesc.Prepare();
@@ -463,17 +450,7 @@ namespace Arc {
 
     std::string delegation_id;
     if(need_delegation) {
-      if(!durl) {
-        logger.msg(INFO, "Failed to delegate credentials to server - no delegation interface found");
-        return false;
-      }
-      AutoPointer<EMIESClient> ac(clients.acquire(durl));
-      delegation_id = ac->delegation();
-      if(delegation_id.empty()) {
-        logger.msg(INFO, "Failed to delegate credentials to server - %s",ac->failure());
-        return false;
-      }
-      clients.release(ac.Release());
+      if(!getDelegationID(durl, delegation_id)) return false;
     }
 
     EMIESResponse *response = NULL;
@@ -495,6 +472,7 @@ namespace Arc {
     delete response;
     
     if(!jobid.manager) jobid.manager = url;
+    if(jobid.delegation_id.empty()) jobid.delegation_id = delegation_id;
   
     // Check if we have anything to upload. Otherwise there is no need to wait.
     if(have_uploads) {
