@@ -25,34 +25,14 @@ namespace Arc {
     return pos != std::string::npos && lower(endpoint.URLString.substr(0, pos)) != "ldap";
   }
 
-  static URL CreateURL(std::string service) {
-    std::string::size_type pos1 = service.find("://");
-    if (pos1 == std::string::npos) {
-      service = "ldap://" + service;
-      pos1 = 4;
-    } else {
-      if(lower(service.substr(0,pos1)) != "ldap") return URL();
-    }
-    std::string::size_type pos2 = service.find(":", pos1 + 3);
-    std::string::size_type pos3 = service.find("/", pos1 + 3);
-    if (pos3 == std::string::npos) {
-      if (pos2 == std::string::npos)
-        service += ":2135";
-      // Is this a good default path?
-      service += "/o=glue";
-    }
-    else if (pos2 == std::string::npos || pos2 > pos3)
-      service.insert(pos3, ":2135");
-
-    return service;
-  }
-
-
   EndpointQueryingStatus TargetInformationRetrieverPluginLDAPGLUE2::Query(const UserConfig& uc, const Endpoint& ce, std::list<ComputingServiceType>& csList, const EndpointQueryOptions<ComputingServiceType>&) const {
     EndpointQueryingStatus s(EndpointQueryingStatus::FAILED);
 
 
-    URL url(CreateURL(ce.URLString));
+    if (isEndpointNotSupported(ce)) {
+      return s;
+    }
+    URL url((ce.URLString.find("://") == std::string::npos ? "ldap://" : "") + ce.URLString, false, 2135, "/o=glue");
     url.ChangeLDAPScope(URL::subtree);
     url.ChangeLDAPFilter("(&(!(GLUE2GroupID=ComputingActivities))(!(ObjectClass=GLUE2ComputingActivity)))");
 
