@@ -30,33 +30,14 @@ namespace Arc {
     return pos != std::string::npos && lower(endpoint.URLString.substr(0, pos)) != "ldap";
   }
 
-  static URL CreateURL(std::string service) {
-    std::string::size_type pos1 = service.find("://");
-    if (pos1 == std::string::npos) {
-      service = "ldap://" + service;
-      pos1 = 4;
-    } else {
-      if(lower(service.substr(0,pos1)) != "ldap") return URL();
-    }
-    std::string::size_type pos2 = service.find(":", pos1 + 3);
-    std::string::size_type pos3 = service.find("/", pos1 + 3);
-    if (pos3 == std::string::npos) {
-      if (pos2 == std::string::npos)
-        service += ":2135";
-      service += "/Mds-Vo-name=local,o=Grid";
-    }
-    else if (pos2 == std::string::npos || pos2 > pos3)
-      service.insert(pos3, ":2135");
-
-    return service;
-  }
-
-
   EndpointQueryingStatus TargetInformationRetrieverPluginLDAPNG::Query(const UserConfig& uc, const Endpoint& cie, std::list<ComputingServiceType>& csList, const EndpointQueryOptions<ComputingServiceType>&) const {
     EndpointQueryingStatus s(EndpointQueryingStatus::FAILED);
 
+    if (isEndpointNotSupported(cie)) {
+      return s;
+    }
     //Query ARIS for all relevant information
-    URL url(CreateURL(cie.URLString));
+    URL url((cie.URLString.find("://") == std::string::npos ? "ldap://" : "") + cie.URLString, false, 2135, "/Mds-Vo-name=local,o=Grid");
     url.ChangeLDAPScope(URL::subtree);
     if (!url) {
       return s;
@@ -358,9 +339,9 @@ namespace Arc {
         for (XMLNode n = (*it)["nordugrid-cluster-benchmark"]; n; ++n) {
           std::string benchmark = (std::string)n;
           std::string::size_type alpha = benchmark.find_first_of("@");
-          std::string benchmarkname = benchmark.substr(0, alpha);
+          std::string benchmarkname = trim(benchmark.substr(0, alpha));
           double performance = 0;
-          stringto(benchmark.substr(alpha + 1), performance);
+          stringto(trim(benchmark.substr(alpha + 1)), performance);
           (*ComputingManager.Benchmarks)[benchmarkname] = performance;
         }
       }
