@@ -5,6 +5,8 @@
 
 #include "SRMURL.h"
 
+namespace ArcDMCSRM {
+
 /**
  * Info about a particular entry in the SRM info file
  */
@@ -36,4 +38,6 @@ class SRMInfo {
   std::string srm_info_filename;
 };
 
+} //namespace ArcDMCSRM
+
 #endif /*SRM_INFO_H_*/
