@@ -522,6 +522,10 @@ namespace Arc {
       info->cookies.push_back(*i);
     }
     info->location = repmsg.Attributes()->get("HTTP:location");
+    // Put all headers in generic map
+    for(AttributeIterator i = repmsg.Attributes()->getAll();i.hasMore();++i) {
+      info->headers.insert(std::pair<std::string, std::string>(i.key(), *i));
+    }
     *response = repmsg.Payload();
     return r;
   }
