@@ -42,15 +42,17 @@ namespace Arc {
   }
   
   static void serialiseJob(const Job& j, Dbt& data) {
-    const std::string version = "3.0.0";
-    const unsigned nItems = 14;
+    const std::string version = "3.0.1";
+    const std::string empty_string;
     const std::string dataItems[] =
       {version, j.IDFromEndpoint, j.Name,
        j.JobStatusInterfaceName, j.JobStatusURL.fullstr(),
        j.JobManagementInterfaceName, j.JobManagementURL.fullstr(),
        j.ServiceInformationInterfaceName, j.ServiceInformationURL.fullstr(),
        j.SessionDir.fullstr(), j.StageInDir.fullstr(), j.StageOutDir.fullstr(),
-       j.JobDescriptionDocument, tostring(j.LocalSubmissionTime.GetTime())};
+       j.JobDescriptionDocument, tostring(j.LocalSubmissionTime.GetTime()),
+       j.DelegationID.size()>0?*j.DelegationID.begin():empty_string};
+    const unsigned nItems = sizeof(dataItems)/sizeof(dataItems[0]);
       
     data.set_data(NULL); data.set_size(0);
     uint32_t l = 0;
@@ -71,7 +73,7 @@ namespace Arc {
     
     std::string version;
     d = parse_string(version, d, size);
-    if (version == "3.0.0") {
+    if ((version == "3.0.0") || (version == "3.0.1")) {
       /* Order of items in record. Version 3.0.0
           {version, j.IDFromEndpoint, j.Name,
            j.JobStatusInterfaceName, j.JobStatusURL.fullstr(),
@@ -79,6 +81,8 @@ namespace Arc {
            j.ServiceInformationInterfaceName, j.ServiceInformationURL.fullstr(),
            j.SessionDir.fullstr(), j.StageInDir.fullstr(), j.StageOutDir.fullstr(),
            j.JobDescriptionDocument, tostring(j.LocalSubmissionTime.GetTime())};
+         Version 3.0.1
+           ..., j.DelegationID}
        */
       std::string s;
       d = parse_string(j.IDFromEndpoint, d, size);
@@ -94,6 +98,11 @@ namespace Arc {
       d = parse_string(s, d, size); j.StageOutDir = URL(s);
       d = parse_string(j.JobDescriptionDocument, d, size);
       d = parse_string(s, d, size); j.LocalSubmissionTime.SetTime(stringtoi(s));
+      j.DelegationID.clear();
+      if (version == "3.0.1") {
+        d = parse_string(s, d, size);
+        if(!s.empty()) j.DelegationID.push_back(s);
+      }
     }
   }
   
@@ -106,7 +115,7 @@ namespace Arc {
     
     std::string version;
     d = parse_string(version, d, size);
-    if (version == "3.0.0") {
+    if ((version == "3.0.0") || (version == "3.0.1")) {
       for (unsigned i = 0; i < n-1; ++i) {
         d = parse_string(attr, d, size);
       }
@@ -161,7 +170,7 @@ namespace Arc {
     dbEnv = new DbEnv(DB_CXX_NO_EXCEPTIONS);
     dbEnv->set_errcall(&handleError);
 
-    if ((ret = dbEnv->open(NULL, DB_CREATE | DB_INIT_CDB | DB_INIT_MPOOL, 0)) != 0) {
+    if ((ret = dbEnv->open(Glib::get_tmp_dir().c_str(), DB_CREATE | DB_INIT_CDB | DB_INIT_MPOOL, 0)) != 0) {
       tearDown();
       throw BDBException(IString("Unable to create data base environment (%s)", name).str(), ret);
     }
@@ -184,19 +193,20 @@ namespace Arc {
       throw BDBException(IString("Unable to set duplicate flags for secondary key DB (%s)", name).str(), ret);
     }
 
-    if ((ret = jobDB->open(NULL, name.c_str(), "job_records", type, flags, 0)) != 0) {
+    std::string absPathToDB = URL(name).Path();
+    if ((ret = jobDB->open(NULL, absPathToDB.c_str(), "job_records", type, flags, 0)) != 0) {
       tearDown();
       throw BDBException(IString("Unable to create job database (%s)", name).str(), ret);
     }
-    if ((ret = nameSecondaryKeyDB->open(NULL, name.c_str(), "name_keys", type, flags, 0)) != 0) {
+    if ((ret = nameSecondaryKeyDB->open(NULL, absPathToDB.c_str(), "name_keys", type, flags, 0)) != 0) {
       tearDown();
       throw BDBException(IString("Unable to create DB for secondary name keys (%s)", name).str(), ret);
     }
-    if ((ret = endpointSecondaryKeyDB->open(NULL, name.c_str(), "endpoint_keys", type, flags, 0)) != 0) {
+    if ((ret = endpointSecondaryKeyDB->open(NULL, absPathToDB.c_str(), "endpoint_keys", type, flags, 0)) != 0) {
       tearDown();
       throw BDBException(IString("Unable to create DB for secondary endpoint keys (%s)", name).str(), ret);
     }
-    if ((ret = serviceInfoSecondaryKeyDB->open(NULL, name.c_str(), "serviceinfo_keys", type, flags, 0)) != 0) {
+    if ((ret = serviceInfoSecondaryKeyDB->open(NULL, absPathToDB.c_str(), "serviceinfo_keys", type, flags, 0)) != 0) {
       tearDown();
       throw BDBException(IString("Unable to create DB for secondary service info keys (%s)", name).str(), ret);
     }
@@ -241,7 +251,7 @@ namespace Arc {
     delete dbEnv; dbEnv = NULL;
 
     dbEnv = new DbEnv(DB_CXX_NO_EXCEPTIONS);
-    dbEnv->remove(NULL, 0);
+    dbEnv->remove(Glib::get_tmp_dir().c_str(), 0);
     delete dbEnv; dbEnv = NULL;
   }
 
@@ -249,7 +259,11 @@ namespace Arc {
     tearDown();
   }
 
+#if ((DB_VERSION_MAJOR > 4)||(DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3))
   void JobInformationStorageBDB::JobDB::handleError(const DbEnv *dbenv, const char *errpfx, const char *msg) {
+#else
+  void JobInformationStorageBDB::JobDB::handleError(const char *errpfx, char *msg) {
+#endif
     if (errpfx) {
       JobInformationStorageBDB::logger.msg(DEBUG, "Error from BDB: %s: %s", errpfx, msg);
     }
