@@ -48,9 +48,14 @@ namespace Arc {
       default_min_speed_time(0),
       default_min_average_speed(0),
       default_max_inactivity_time(300),
-      show_progress(NULL) {}
+      show_progress(NULL),
+      cancelled(false) {}
 
-  DataMover::~DataMover() {}
+  DataMover::~DataMover() {
+    Cancel();
+    // Wait for Transfer() to finish with lock
+    Glib::Mutex::Lock lock(lock_);
+  }
 
   bool DataMover::verbose() {
     return be_verbose;
@@ -97,6 +102,7 @@ namespace Arc {
   } transfer_struct;
 
   DataStatus DataMover::Delete(DataPoint& url, bool errcont) {
+    DataStatus res = DataStatus::Success;
     bool remove_lfn = !url.HaveLocations(); // pfn or plain url
     if (!url.Resolve(true).Passed())
       // TODO: Check if error is real or "not exist".
@@ -123,8 +129,10 @@ namespace Arc {
         }
         else {
           url.SetSecure(false);
-          if (!url.Remove()) {
+          DataStatus r = url.Remove();
+          if (!r) {
             logger.msg(INFO, "Failed to delete physical file");
+            res = r;
             if (!errcont) {
               url.NextLocation();
               continue;
@@ -136,9 +144,10 @@ namespace Arc {
         if (url.IsIndex()) {
           logger.msg(INFO, "Removing metadata in %s",
                      url.CurrentLocationMetadata());
-          DataStatus err = url.Unregister(false);
-          if (!err) {
-            logger.msg(ERROR, "Failed to delete meta-information");
+          DataStatus r = url.Unregister(false);
+          if (!r) {
+            logger.msg(INFO, "Failed to delete meta-information");
+            res = r;
             url.NextLocation();
           }
           else
@@ -151,22 +160,22 @@ namespace Arc {
       }
     if (url.IsIndex()) {
       if (url.HaveLocations()) {
-        logger.msg(ERROR, "Failed to remove all physical instances");
-        return DataStatus::DeleteError;
+        logger.msg(INFO, "Failed to remove all physical instances");
+        return res;
       }
       if (remove_lfn) {
         logger.msg(INFO, "Removing logical file from metadata %s", url.str());
-        DataStatus err = url.Unregister(true);
-        if (!err) {
-          logger.msg(ERROR, "Failed to delete logical file");
-          return err;
+        DataStatus r = url.Unregister(true);
+        if (!r) {
+          logger.msg(INFO, "Failed to delete logical file");
+          return r;
         }
       }
     }
     else {
       if (!url.LocationValid()) {
-        logger.msg(ERROR, "Failed to remove instance");
-        return DataStatus::DeleteError;
+        logger.msg(INFO, "Failed to remove instance");
+        return res;
       }
     }
     return DataStatus::Success;
@@ -246,12 +255,12 @@ namespace Arc {
     if (!source) {
       logger.msg(ERROR, "Not valid source");
       source.NextTry();
-      return DataStatus::ReadAcquireError;
+      return DataStatus(DataStatus::ReadAcquireError, EINVAL, "Source is not valid");
     }
     if (!destination) {
       logger.msg(ERROR, "Not valid destination");
       destination.NextTry();
-      return DataStatus::WriteAcquireError;
+      return DataStatus(DataStatus::WriteAcquireError, EINVAL, "Destination is not valid");
     }
     // initial cache check, if the DN is cached we can exit straight away
     bool cacheable = false;
@@ -310,7 +319,7 @@ namespace Arc {
         if (source.HaveLocations())
           break;
         logger.msg(ERROR, "No locations for source found: %s", source.str());
-        dres = DataStatus::ReadResolveError;
+        dres = DataStatus(DataStatus::ReadResolveError, EARCRESINVAL, "No locations found");
       }
       else
         logger.msg(ERROR, "Failed to resolve source: %s", source.str());
@@ -327,7 +336,7 @@ namespace Arc {
           break;
         logger.msg(ERROR, "No locations for destination found: %s",
                    destination.str());
-        dres = DataStatus::WriteResolveError;
+        dres = DataStatus(DataStatus::WriteResolveError, EARCRESINVAL, "No locations found");
       }
       else
         logger.msg(ERROR, "Failed to resolve destination: %s", destination.str());
@@ -347,7 +356,7 @@ namespace Arc {
         if (!destination.HaveLocations()) {
           logger.msg(ERROR, "No locations for destination different from source "
                      "found: %s", destination.str());
-          return DataStatus::WriteResolveError;
+          return DataStatus(DataStatus::WriteResolveError, EEXIST);
         }
       }
     // Try to avoid any additional checks meant to provide
@@ -418,6 +427,7 @@ namespace Arc {
     DataStatus res = DataStatus::TransferError;
     int try_num;
     for (try_num = 0;; try_num++) { /* cycle for retries */
+      Glib::Mutex::Lock lock(lock_);
       logger.msg(VERBOSE, "DataMover: cycle");
       if ((try_num != 0) && (!do_retries)) {
         logger.msg(VERBOSE, "DataMover: no retries requested - exit");
@@ -566,7 +576,7 @@ namespace Arc {
             if (is_locked) {
               logger.msg(VERBOSE, "Cached file is locked - should retry");
               source.NextTry(); /* to decrease retry counter */
-              return DataStatus::CacheErrorRetryable;
+              return DataStatus(DataStatus::CacheError, EAGAIN, "Cache file locked");
             }
             cacheable = false;
             logger.msg(INFO, "Failed to initiate cache");
@@ -579,10 +589,12 @@ namespace Arc {
                        canonic_url, cache.File(canonic_url));
             // check the list of cached DNs
             bool have_permission = false;
+            // don't request metadata from source if user says not to
+            bool check_meta = (source.GetURL().Option("cache") != "invariant");
             if (source.GetURL().Option("cache") != "check" && cache.CheckDN(canonic_url, dn))
               have_permission = true;
             else {
-              DataStatus cres = source.Check();
+              DataStatus cres = source.Check(check_meta);
               if (!cres.Passed()) {
                 logger.msg(ERROR, "Permission checking failed: %s", canonic_url);
                 source.NextLocation(); /* try another source */
@@ -595,12 +607,12 @@ namespace Arc {
             logger.msg(INFO, "Permission checking passed");
             /* check if file is fresh enough */
             bool outdated = true;
-            if (have_permission)
+            if (have_permission || !check_meta)
               outdated = false; // cached DN means don't check creation date
-            if (source.CheckCreated() && cache.CheckCreated(canonic_url)) {
-              Time sourcetime = source.GetCreated();
+            if (source.CheckModified() && cache.CheckCreated(canonic_url)) {
+              Time sourcetime = source.GetModified();
               Time cachetime = cache.GetCreated(canonic_url);
-              logger.msg(VERBOSE, "Source creation date: %s", sourcetime.str());
+              logger.msg(VERBOSE, "Source modification date: %s", sourcetime.str());
               logger.msg(VERBOSE, "Cache creation date: %s", cachetime.str());
               if (sourcetime <= cachetime)
                 outdated = false;
@@ -630,7 +642,7 @@ namespace Arc {
               source.NextTry(); /* to decrease retry counter */
               if (is_locked) {
                 logger.msg(VERBOSE, "Cached file is locked - should retry");
-                return DataStatus::CacheErrorRetryable;
+                return DataStatus(DataStatus::CacheError, EAGAIN, "Cache file locked");
               }
               return DataStatus::CacheError;
             }
@@ -649,7 +661,7 @@ namespace Arc {
           /* check permissions first */
           logger.msg(INFO, "URL is mapped to local access - "
                      "checking permissions on original URL");
-          DataStatus cres =  source.Check();
+          DataStatus cres =  source.Check(false);
           if (!cres.Passed()) {
             logger.msg(ERROR, "Permission checking on original URL failed: %s",
                        source.str());
@@ -673,24 +685,28 @@ namespace Arc {
               std::string dirpath = Glib::path_get_dirname(link_name);
               if(dirpath == ".") dirpath = G_DIR_SEPARATOR_S;
               if (!DirCreate(dirpath, user.get_uid(), user.get_gid(), S_IRWXU, true) != 0) {
+                int err = errno;
                 logger.msg(ERROR, "Failed to create directory %s", dirpath);
                 source.NextLocation(); /* try another source */
                 logger.msg(VERBOSE, "source.next_location");
-                res = DataStatus::ReadStartError;
+                res = DataStatus(DataStatus::ReadStartError, err);
                 continue;
               }
             }
             // make link
             if (symlink(file_name.c_str(), link_name.c_str()) == -1) {
+              int err = errno;
               logger.msg(ERROR, "Failed to make symbolic link %s to %s : %s",
                          link_name, file_name, StrError());
               source.NextLocation(); /* try another source */
               logger.msg(VERBOSE, "source.next_location");
-              res = DataStatus::ReadStartError;
+              res = DataStatus(DataStatus::ReadStartError, err);
               continue;
             }
             User user;
-            (lchown(link_name.c_str(), user.get_uid(), user.get_gid()) != 0);
+            if (lchown(link_name.c_str(), user.get_uid(), user.get_gid()) == -1) {
+              logger.msg(WARNING, "Failed to change owner of symbolic link %s to %i", link_name, user.get_uid());
+            }
             return DataStatus::Success;
             // Leave after making a link. Rest moves data.
           }
@@ -730,7 +746,7 @@ namespace Arc {
           logger.msg(ERROR, "Failed to stat source %s", source_url.str());
           if (source.NextLocation())
             logger.msg(VERBOSE, "(Re)Trying next source");
-          res = DataStatus::ReadError;
+          res = r;
 #ifndef WIN32
           if (cacheable)
             cache.StopAndDelete(canonic_url);
@@ -745,7 +761,7 @@ namespace Arc {
             logger.msg(ERROR, "Meta info of source and location do not match for %s", source_url.str());
           if (source.NextLocation())
             logger.msg(VERBOSE, "(Re)Trying next source");
-          res = DataStatus::ReadError;
+          res = r;
 #ifndef WIN32
           if (cacheable)
             cache.StopAndDelete(canonic_url);
@@ -916,7 +932,8 @@ namespace Arc {
 #endif
       }
       logger.msg(VERBOSE, "Waiting for buffer");
-      for (; (!buffer.eof_read() || !buffer.eof_write()) && !buffer.error();) {
+      // cancelling will make loop exit before eof, triggering error and destinatinon cleanup
+      for (; (!buffer.eof_read() || !buffer.eof_write()) && !buffer.error() && !cancelled;) {
         buffer.wait_any();
         if (cacheable && !cache_lock.empty()) {
           // touch cache lock file regularly so it is still valid
@@ -956,6 +973,12 @@ namespace Arc {
                                        destination_meta_initially_stored).Passed())
           logger.msg(ERROR, "Failed to unregister preregistered lfn. "
                      "You may need to unregister it manually");
+
+        // Check for cancellation
+        if (cancelled) {
+          logger.msg(INFO, "Transfer cancelled successfully");
+          return DataStatus::SuccessCancelled;
+        }
         // Analyze errors
         // Easy part first - if either read or write part report error
         // go to next endpoint.
@@ -981,7 +1004,7 @@ namespace Arc {
         else if (buffer.error_transfer()) {
           // Here is more complicated case - operation timeout
           // Let's first check if buffer was full
-          res = DataStatus::TransferError;
+          res = DataStatus(DataStatus::TransferError, ETIMEDOUT);
           if (!buffer.for_read()) {
             // No free buffers for 'read' side. Buffer must be full.
             res.SetDesc(destination.GetFailureReason().GetDesc());
@@ -1051,7 +1074,7 @@ namespace Arc {
           if (!destination.Unregister(replication || destination_meta_initially_stored)) {
             logger.msg(WARNING, "Failed to unregister preregistered lfn, You may need to unregister it manually");
           }
-          res = DataStatus(DataStatus::TransferError, "Checksum mismatch");
+          res = DataStatus(DataStatus::TransferError, EARCCHECKSUM);
           if (!Delete(destination, true)) {
             logger.msg(WARNING, "Failed to delete destination, retry may fail");
           }
@@ -1064,7 +1087,12 @@ namespace Arc {
           std::string src_csum_s(source.GetCheckSum());
           if (src_csum_s.find(':') == src_csum_s.length() -1) {
             logger.msg(VERBOSE, "Cannot compare empty checksum");
-          } else if (calc_csum.substr(0, calc_csum.find(":")) != src_csum_s.substr(0, src_csum_s.find(":"))) {
+          }
+          // Check the checksum types match. Some buggy GridFTP servers return a
+          // different checksum type than requested so also check that the checksum
+          // length matches before comparing.
+          else if (calc_csum.substr(0, calc_csum.find(":")) != src_csum_s.substr(0, src_csum_s.find(":")) ||
+                   calc_csum.substr(calc_csum.find(":")).length() != src_csum_s.substr(src_csum_s.find(":")).length()) {
             logger.msg(VERBOSE, "Checksum type of source and calculated checksum differ, cannot compare");
           } else if (calc_csum.substr(calc_csum.find(":")) != src_csum_s.substr(src_csum_s.find(":"))) {
             logger.msg(ERROR, "Checksum mismatch between calcuated checksum %s and source checksum %s", calc_csum, source.GetCheckSum());
@@ -1073,7 +1101,7 @@ namespace Arc {
               cache.StopAndDelete(canonic_url);
             }
 #endif
-            res = DataStatus::TransferError;
+            res = DataStatus(DataStatus::TransferError, EARCCHECKSUM);
             if (source.NextLocation()) {
               logger.msg(VERBOSE, "(Re)Trying next source");
             }
@@ -1127,7 +1155,7 @@ namespace Arc {
             logger.msg(ERROR, "Failed to unregister preregistered lfn. "
                        "You may need to unregister it manually");
           source.NextTry(); /* to decrease retry count */
-          if (is_locked) return DataStatus::CacheErrorRetryable;
+          if (is_locked) return DataStatus(DataStatus::CacheError, EAGAIN, "Cache file locked");
           return DataStatus::CacheError; // retry won't help
         }
       }
@@ -1139,6 +1167,10 @@ namespace Arc {
     return DataStatus::Success;
   }
 
+  void DataMover::Cancel() {
+    cancelled = true;
+  }
+
   void DataMover::secure(bool val) {
     force_secure = val;
   }
