@@ -420,8 +420,8 @@ Arc::MCC_Status ARexService::make_http_fault(Arc::Message& outmsg,int code,const
   Arc::PayloadRaw* outpayload = new Arc::PayloadRaw();
   outmsg.Payload(outpayload);
   outmsg.Attributes()->set("HTTP:CODE",Arc::tostring(code));
-  if(resp) outmsg.Attributes()->set("HTTP:RESPONSE",resp);
-  return Arc::MCC_Status(Arc::STATUS_OK);
+  if(resp) outmsg.Attributes()->set("HTTP:REASON",resp);
+  return Arc::MCC_Status(Arc::UNKNOWN_SERVICE_ERROR);
 }
 
 Arc::MCC_Status ARexService::make_fault(Arc::Message& /*outmsg*/) {
@@ -982,7 +982,7 @@ ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Servic
     if (!lrms_name_.empty()) {
       logger_.msg(Arc::ERROR, "Provided LRMSName is not a valid URL: %s",lrms_name_);
     } else {
-      logger_.msg(Arc::WARNING, "No LRMSName is provided. This is needed if you wish to completely comply with the BES specifications.");
+      logger_.msg(Arc::VERBOSE, "No LRMSName is provided. This is needed if you wish to completely comply with the BES specifications.");
     };
     // Filling something to make it follow BES specs
     lrms_name_ = "uri:undefined";
@@ -991,7 +991,7 @@ ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Servic
   os_name_ = (std::string)((*cfg)["OperatingSystem"]);
   std::string debugLevel = (std::string)((*cfg)["debugLevel"]);
   if(!debugLevel.empty()) {
-    logger_.setThreshold(Arc::string_to_level(debugLevel));
+    logger_.setThreshold(Arc::istring_to_level(debugLevel));
   };
   int valuei;
   if ((!(*cfg)["InfoproviderWakeupPeriod"]) ||
