@@ -4,6 +4,8 @@
 
 #include <arc/message/SOAPEnvelope.h>
 #include <arc/ws-addressing/WSA.h>
+#include "delegation/DelegationStores.h"
+#include "delegation/DelegationStore.h"
 #include "job.h"
 #include "grid-manager/files/ControlFileHandling.h"
 
@@ -459,6 +461,26 @@ Arc::MCC_Status ARexService::ESGetActivityInfo(ARexGMConfig& config,Arc::XMLNode
               status = std::string("emies:")+primary_state;
             };
             */
+            // TODO: all additional ellements can be stored into xml generated by infoprovider
+            // Extensions store delegation id(s)
+            std::list<std::string> delegation_ids;
+            DelegationStores* deleg = config.GmConfig().Delegations();
+            if(deleg) {
+              DelegationStore& dstore = (*deleg)[config.GmConfig().DelegationDir()];
+              delegation_ids = dstore.ListLockedCredIDs(jobid, config.GridName());
+            };
+            if(!delegation_ids.empty()) {
+              Arc::XMLNode extensions = info.NewChild(glue2_prefix+":Extensions");
+              int n = 0;
+              for(std::list<std::string>::iterator id = delegation_ids.begin();
+                              id != delegation_ids.end();++id) {
+                Arc::XMLNode extension = extensions.NewChild(glue2_prefix+":Extension");
+                extension.NewChild(glue2_prefix+":LocalID") = "urn:delegid:nordugrid.org";
+                extension.NewChild(glue2_prefix+":Key") = Arc::tostring(n);
+                extension.NewChild(glue2_prefix+":Value") = *id;
+                // TODO: add source and destination later
+              };
+            };
             // Additional elements
             info.NewChild("esainfo:StageInDirectory") = config.Endpoint()+"/"+job.ID();
             info.NewChild("esainfo:StageOutDirectory") = config.Endpoint()+"/"+job.ID();
