@@ -187,6 +187,14 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
         config.job_log->SetReporter(url.c_str());
       }
     }
+    else if (command == "jobreport_vo_filters") { // which VO will be send to the server
+      if (!config.job_log) continue;
+      for(;;) {
+        std::string voFilters = config_next_arg(rest);
+        if (voFilters.empty()) break;
+        config.job_log->SetVoFilters(voFilters.c_str());
+      }
+    }
     else if (command == "jobreport_publisher") { // Name of the publisher: e.g. jura, arc-ur-logger
       if (!config.job_log) continue;
       std::string publisher = config_next_arg(rest);
@@ -281,6 +289,14 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
         logger.msg(Arc::ERROR, "Wrong number in maxtransfertries"); return false;
       }
     }
+    else if (command == "acix_endpoint")  {
+      std::string endpoint(config_next_arg(rest));
+      if (!Arc::URL(endpoint) || endpoint.find("://") == std::string::npos) {
+        logger.msg(Arc::ERROR, "Bad URL in acix_endpoint"); return false;
+      }
+      endpoint.replace(0, endpoint.find("://"), "acix");
+      config.acix_endpoint = endpoint;
+    }
     else if(command == "norootpower") {
       if (!CheckYesNoCommand(config.strict_session, command, rest)) return false;
     }
@@ -548,6 +564,7 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
     passiveTransfer
     localTransfer
     preferredPattern
+    acixEndpoint
     timeouts
       minSpeed
       minSpeedTime
@@ -580,6 +597,14 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
     if (!elementtobool(tmp_node, "localTransfer", config.use_local_transfer, &logger)) return false;
     if (!elementtoint(tmp_node, "maxRetries", config.max_retries, &logger)) return false;
     if (tmp_node["preferredPattern"]) config.preferred_pattern = (std::string)(tmp_node["preferredPattern"]);
+    if (tmp_node["acixEndpoint"]) {
+      std::string endpoint((std::string)(tmp_node["acixEndPoint"]));
+      if (!Arc::URL(endpoint) || endpoint.find("://") == std::string::npos) {
+        logger.msg(Arc::ERROR, "Bad URL in acix_endpoint"); return false;
+      }
+      endpoint.replace(0, endpoint.find("://"), "acix");
+      config.acix_endpoint = endpoint;
+    }
   }
   /*
   serviceMail
