@@ -75,7 +75,7 @@ sub type_and_version() {
 sub collect_node_data() {
     return if $allnodedata_initialized;
     $allnodedata_initialized = 1;
-    my ($out, $err, $ret) = condor_run("condor_status -long");
+    my ($out, $err, $ret) = condor_run('condor_status -format "Name = %V\n" Name -format "Machine = %V\n" Machine -format "State = %V\n" State -format "Cpus = %V\n" Cpus -format "TotalCpus = %V\n" TotalCpus -format "SlotType = %V\n\n" SlotType');
     error("Failed collecting node information.") if $ret;
     for (split /\n\n+/, $out) {
         my %target = condor_digest_classad($_);
@@ -91,7 +91,7 @@ sub collect_node_data() {
 sub collect_job_data() {
     return if $alljobdata_initialized;
     $alljobdata_initialized = 1;
-    my ($out, $err, $ret) = condor_run("condor_q -long -global -constraint 'NiceUser == False'");
+    my ($out, $err, $ret) = condor_run('condor_q -constraint "NiceUser == False" -format "ClusterId = %V\n" ClusterId -format "ProcId = %V\n" ProcId -format "JobStatus = %V\n" JobStatus -format "CurrentHosts = %V\n" CurrentHosts -format "LastRemoteHost = %V\n" LastRemoteHost -format "RemoteHost = %V\n" RemoteHost -format "ImageSize = %V\n" ImageSize -format "RemoteWallClockTime = %V\n" RemoteWallClockTime -format "RemoteUserCpu = %V\n" RemoteUserCpu -format "RemoteSysCpu = %V\n" RemoteSysCpu -format "JobTimeLimit = %V\n" JobTimeLimit -format "JobCpuLimit = %V\n\n" JobCpuLimit');
     return if $out =~ m/All queues are empty/;
     error("Failed collecting job information.") if $ret;
     for (split /\n\n+/, $out) {
@@ -126,7 +126,7 @@ sub collect_jobids($$) {
     close LOCAL;
     foreach my $pair (values %pairs) {
         # get rid of .condor from localid.
-        $$pair{id} =~ s/(.*)\.condor$/$1.0/;
+        $$pair{id} =~ s/(\d+)\..*/$1.0/;
         if ( $$pair{queue} eq $qname ) {
             push @jobids_thisqueue, $$pair{id};
         } else {
@@ -300,16 +300,17 @@ sub condor_cluster_get_running_jobs() {
 # interactively by their owners
 #
 sub condor_queue_get_running() {
-    my @free;
+    my $running = 0;
     my @qnod = condor_queue_get_nodes();
-    for (@allnodedata) {
+
+    for (@allnodedata)
+    {
         my %node = %$_;
-        next unless grep { $node{name} =~ /^((vm|slot)\d+@)?$_/i } @qnod;
-        next unless $node{state} =~ /^Unclaimed/i;
-        push @free, $node{name};
+        next unless grep { $_ eq $node{machine} } @qnod;
+        $running += $node{cpus} if ($node{slottype} !~ /^Partitionable/i && $node{state} !~ /^Unclaimed/i);
     }
-    my $running = @qnod - @free;
-    debug "===condor_queue_get_running: $running; free nodes: ". scalar @free ." (@free)";
+
+    debug "===condor_queue_get_running: $running";
     return $running;
 }
 
@@ -317,18 +318,51 @@ sub condor_queue_get_running() {
 # Same as above, but for the whole cluster
 #
 sub condor_cluster_get_usedcpus() {
-    my $free = 0;
-    do {$free += 1 if $$_{state} =~ /^Unclaimed/i} for @allnodedata;
-    my $used = condor_cluster_totalcpus() - $free;
+    my $used = 0;
+    for (@allnodedata)
+    {
+       $used += $$_{cpus} if ($$_{slottype} !~ /^Partitionable/i && $$_{state} !~ /^Unclaimed/i);
+    }
     debug "===condor_cluster_get_usedcpus: $used";
     return $used;
 }
 
+#
+# returns the total number of CPUs in the cluster
+#
+sub condor_queue_totalcpus() {
+    my @qnod = condor_queue_get_nodes();
+
+    # List all machines in the pool. Create a hash specifying the TotalCpus
+    # for each machine.
+    my %machines;
+    $machines{$$_{machine}} = $$_{totalcpus} for @allnodedata;
+
+    my $totalcpus = 0;
+    for (keys %machines) {
+        my $machine = $_;
+        next unless grep { $machine eq $_ } @qnod;
+        $totalcpus += $machines{$_};
+    }
+
+    return $totalcpus;
+}
+
 #
 # returns the total number of nodes in the cluster
 #
 sub condor_cluster_totalcpus() {
-    return scalar @allnodedata;
+    # List all machines in the pool. Create a hash specifying the TotalCpus
+    # for each machine.
+    my %machines;
+    $machines{$$_{machine}} = $$_{totalcpus} for @allnodedata;
+
+    my $totalcpus = 0;
+    for (keys %machines) {
+        $totalcpus += $machines{$_};
+    }
+
+    return $totalcpus;
 }
 
 #
@@ -360,11 +394,10 @@ sub condor_job_suspended($) {
 # CPU distribution string (e.g., '1cpu:5 2cpu:1').
 #
 sub cpudistribution {
-    # List all machines in the pool.  Machines with multiple CPUs are listed
-    # one time for each CPU, with a prefix such as 'slot1@', 'slot2@', etc.
-    # (or 'vm1@', 'vm2@' in older Condor releases)
+    # List all machines in the pool. Create a hash specifying the TotalCpus
+    # for each machine.
     my %machines;
-    $machines{$$_{machine}}++ for @allnodedata;
+    $machines{$$_{machine}} = $$_{totalcpus} for @allnodedata;
 
     # Count number of machines with one CPU, number with two, etc.
     my %dist;
@@ -410,7 +443,7 @@ sub cluster_info ($) {
     # Note: SGE has the concept of "slots", which roughly corresponds to
     # concept of "cpus" in ARC (PBS) LRMS interface.
 
-    $lrms_cluster{totalcpus} = scalar @allnodedata;
+    $lrms_cluster{totalcpus} = condor_cluster_totalcpus();
     $lrms_cluster{cpudistribution} = cpudistribution();
     $lrms_cluster{usedcpus} = condor_cluster_get_usedcpus();
     #NOTE: counts jobs, not cpus.
@@ -448,7 +481,7 @@ sub queue_info ($$) {
 
     # Number of available (free) cpus can not be larger that
     # free cpus in the whole cluster
-    my $totalcpus = scalar condor_queue_get_nodes();
+    my $totalcpus = condor_queue_totalcpus();
     my $usedcpus = condor_queue_get_running();
     my $queuedcpus = condor_queue_get_queued();
 
@@ -469,7 +502,7 @@ sub queue_info ($$) {
     # nordugrid-queue-maxuserrun
     # nordugrid-queue-mincputime
     # nordugrid-queue-defaultcputime
-    $lrms_queue{maxrunning} = scalar condor_queue_get_nodes();
+    $lrms_queue{maxrunning} = $totalcpus;
     $lrms_queue{maxqueuable} = 2 * $lrms_queue{maxrunning};
     $lrms_queue{maxuserrun} = $lrms_queue{maxrunning};
 
@@ -497,10 +530,10 @@ sub jobs_info ($$@) {
 
     foreach my $id ( @$jids ) {
         # submit-condor-job might return identifiers of the form ClusterId.condor
-        # Replace .condor with .0. It is safe to assume that ProcId is 0 because
+        # Replace .hostname with .0. It is safe to assume that ProcId is 0 because
         # we only submit one job at a time.
         my $id0 = $id;
-        $id0 =~ s/(.*)\.condor$/$1.0/;
+        $id0 =~ s/(\d+)\..*/$1.0/;
         debug "===jobs_info: Mapping $id to $id0";
 
         if ( $alljobdata{$id0} ) {
@@ -509,12 +542,13 @@ sub jobs_info ($$@) {
             $lrms_jobs{$id}{mem} = $job{lc 'ImageSize'};
             $lrms_jobs{$id}{walltime} = floor($job{lc 'RemoteWallClockTime'} / 60);
             $lrms_jobs{$id}{cputime} = floor(($job{lc 'RemoteUserCpu'} + $job{lc 'RemoteSysCpu'}) / 60);
-            $lrms_jobs{$id}{nodes} = [$job{lc 'LastRemoteHost'}];
-            $lrms_jobs{$id}{nodes} = [$job{lc 'RemoteHost'}] if $job{lc 'RemoteHost'};
-            if (defined $job{lc 'JobTimeLimit'}) {
+            $lrms_jobs{$id}{nodes} = "";
+            $lrms_jobs{$id}{nodes} = [$job{lc 'LastRemoteHost'}] if ($job{lc 'LastRemoteHost'} ne "undefined");
+            $lrms_jobs{$id}{nodes} = [$job{lc 'RemoteHost'}] if ($job{lc 'RemoteHost'} ne "undefined");
+            if ($job{lc 'JobTimeLimit'} ne "undefined") {
                 $lrms_jobs{$id}{reqwalltime} = floor($job{lc 'JobTimeLimit'} / 60); # caller knows these better
             }
-            if (defined $job{lc 'JobCpuLimit'}) {
+            if ($job{lc 'JobCpuLimit'} ne "undefined") {
                 $lrms_jobs{$id}{reqcputime} = floor($job{lc 'JobCpuLimit'} / 60); # caller knows these better
             }
             $lrms_jobs{$id}{rank} = rank($id0) ? rank($id0) : '';
