@@ -26,14 +26,14 @@
 #include <arc/credential/Credential.h>
 #include <arc/ws-addressing/WSA.h>
 
-#include "grid-manager/conf/environment.h"
-#include "grid-manager/conf/conf_pre.h"
-#include "grid-manager/jobs/job.h"
-#include "grid-manager/jobs/plugins.h"
-#include "grid-manager/jobs/job_request.h"
-#include "grid-manager/jobs/commfifo.h"
-#include "grid-manager/run/run_plugin.h"
-#include "grid-manager/files/info_files.h"
+#include "grid-manager/conf/GMConfig.h"
+#include "grid-manager/jobs/GMJob.h"
+#include "grid-manager/jobs/ContinuationPlugins.h"
+#include "grid-manager/jobs/JobDescriptionHandler.h"
+#include "grid-manager/jobs/CommFIFO.h"
+#include "grid-manager/jobs/JobsList.h"
+#include "grid-manager/run/RunPlugin.h"
+#include "grid-manager/files/ControlFileHandling.h"
 #include "delegation/DelegationStores.h"
 #include "delegation/DelegationStore.h"
 
@@ -41,100 +41,33 @@
 
 using namespace ARex;
 
-ARexGMConfig::~ARexGMConfig(void) {
-  if(user_) delete user_;
-}
+Arc::Logger ARexGMConfig::logger(Arc::Logger::getRootLogger(), "ARexGMConfig");
 
-ARexGMConfig::ARexGMConfig(const GMEnvironment& env,const std::string& uname,const std::string& grid_name,const std::string& service_endpoint):user_(NULL),readonly_(false),grid_name_(grid_name),service_endpoint_(service_endpoint) {
+ARexGMConfig::ARexGMConfig(const GMConfig& config,const std::string& uname,const std::string& grid_name,const std::string& service_endpoint):
+    config_(config),user_(uname),readonly_(false),grid_name_(grid_name),service_endpoint_(service_endpoint) {
   //if(!InitEnvironment(configfile)) return;
   // const char* uname = user_s.get_uname();
   //if((bool)job_map) uname=job_map.unix_name();
-  user_=new JobUser(env,uname);
-  if(!user_->is_valid()) { delete user_; user_=NULL; return; };
-  if(env.nordugrid_loc().empty()) { delete user_; user_=NULL; return; };
-  /* read configuration */
-  std::vector<std::string> session_roots;
-  std::string control_dir;
-  std::string default_lrms;
-  std::string default_queue;
-  RunPlugin* cred_plugin = new RunPlugin;
-  std::string allowsubmit;
-  bool strict_session;
-  std::string gridftp_endpoint; // for gridftp interface
-  std::string arex_endpoint; // our interface
-  std::string username = uname;
-  bool enable_arc;
-  bool enable_emies;
-  std::string::size_type p =username.find(':');
-  if(p != std::string::npos) username.resize(p);
-  if(!configure_user_dirs(username,control_dir,session_roots,
-                          session_roots_non_draining_,
-                          default_lrms,default_queue,queues_,
-                          cont_plugins_,*cred_plugin,
-                          allowsubmit,strict_session,
-                          gridftp_endpoint,arex_endpoint,
-                          enable_arc,enable_emies,env)
-                          || control_dir.empty() ) {
-    // olog<<"Failed processing grid-manager configuration"<<std::endl;
-    delete user_; user_=NULL; delete cred_plugin; return;
-  };
-  delete cred_plugin;
-  if(default_queue.empty() && (queues_.size() == 1)) {
-    default_queue=*(queues_.begin());
-  };
-  if(!arex_endpoint.empty()) service_endpoint_ = arex_endpoint;
-  user_->SetControlDir(control_dir);
-  user_->SetSessionRoot(session_roots);
-  user_->SetLRMS(default_lrms,default_queue);
-  user_->SetStrictSession(strict_session);
-  //for(;allowsubmit.length();) {
-  //  std::string group = config_next_arg(allowsubmit);
-  //  if(group.length()) readonly=true;
-  //  if(user_a.check_group(group)) { readonly=false; break; };
-  //};
-  //if(readonly) olog<<"This user is denied to submit new jobs"<<std::endl;
-  /*
-          * link to the class for direct file access *
-          std::string direct_config = "mount "+session_root+"\n";
-          direct_config+="dir / nouser read cd dirlist delete append overwrite";          direct_config+=" create "+
-             inttostring(user->get_uid())+":"+inttostring(user->get_gid())+
-             " 600:600";
-          direct_config+=" mkdir "+
-             inttostring(user->get_uid())+":"+inttostring(user->get_gid())+
-             " 700:700\n";
-          direct_config+="end\n";
-#ifdef HAVE_SSTREAM
-          std::stringstream fake_cfile(direct_config);
-#else
-          std::strstream fake_cfile;
-          fake_cfile<<direct_config;
-#endif
-          direct_fs = new DirectFilePlugin(fake_cfile,user_s);
-          if((bool)job_map) {
-            olog<<"Job submission user: "<<uname<<
-                  " ("<<user->get_uid()<<":"<<user->get_gid()<<")"<<std::endl;
-  */
+  if(!user_) {
+    logger.msg(Arc::WARNING, "Cannot handle local user %s", uname);
+    return;
+  }
+  // Do substitutions on session dirs
+  session_roots_ = config_.SessionRoots();
+  for (std::vector<std::string>::iterator session = session_roots_.begin();
+       session != session_roots_.end(); ++session) {
+    config_.Substitute(*session, user_);
+  }
+  session_roots_non_draining_ = config_.SessionRootsNonDraining();
+  for (std::vector<std::string>::iterator session = session_roots_non_draining_.begin();
+       session != session_roots_non_draining_.end(); ++session) {
+    config_.Substitute(*session, user_);
+  }
+  if(!config_.AREXEndpoint().empty()) service_endpoint_ = config_.AREXEndpoint();
 }
 
-template <typename T> class AutoPointer {
- private:
-  T* object;
-  void operator=(const AutoPointer<T>&) { };
-  void operator=(T*) { };
-  AutoPointer(const AutoPointer&):object(NULL) { };
- public:
-  AutoPointer(void):object(NULL) { };
-  AutoPointer(T* o):object(o) { }
-  ~AutoPointer(void) { if(object) delete object; };
-  T& operator*(void) const { return *object; };
-  T* operator->(void) const { return object; };
-  operator bool(void) const { return (object!=NULL); };
-  bool operator!(void) const { return (object==NULL); };
-  operator T*(void) const { return object; };
-};
-
 static ARexJobFailure setfail(JobReqResult res) {
-  switch(res) {
+  switch(res.result_type) {
     case JobReqSuccess: return ARexJobNoError;
     case JobReqInternalFailure: return ARexJobInternalError;
     case JobReqSyntaxFailure: return ARexJobDescriptionSyntaxError;
@@ -158,16 +91,16 @@ bool ARexJob::is_allowed(bool fast) {
   // Do fine-grained authorization requested by job's owner
   if(config_.beginAuth() == config_.endAuth()) return true;
   std::string acl;
-  if(!job_acl_read_file(id_,*config_.User(),acl)) return true; // safe to ignore
+  if(!job_acl_read_file(id_,config_.GmConfig(),acl)) return true; // safe to ignore
   if(acl.empty()) return true; // No policy defiled - only owner allowed
   // Identify and parse policy
   ArcSec::EvaluatorLoader eval_loader;
-  AutoPointer<ArcSec::Policy> policy(eval_loader.getPolicy(ArcSec::Source(acl)));
+  Arc::AutoPointer<ArcSec::Policy> policy(eval_loader.getPolicy(ArcSec::Source(acl)));
   if(!policy) {
     logger_.msg(Arc::VERBOSE, "%s: Failed to parse user policy", id_);
     return true;
   };
-  AutoPointer<ArcSec::Evaluator> eval(eval_loader.getEvaluator(policy));
+  Arc::AutoPointer<ArcSec::Evaluator> eval(eval_loader.getEvaluator(policy.Ptr()));
   if(!eval) {
     logger_.msg(Arc::VERBOSE, "%s: Failed to load evaluator for user policy ", id_);
     return true;
@@ -207,7 +140,7 @@ bool ARexJob::is_allowed(bool fast) {
     action=JOB_POLICY_OPERATION_MODIFY; action.NewAttribute("Type")="string";
     action.NewAttribute("AttributeId")=JOB_POLICY_OPERATION_URN;
     // Evaluating policy
-    ArcSec::Response *resp = eval->evaluate(request,policy);
+    ArcSec::Response *resp = eval->evaluate(request,policy.Ptr());
     // Analyzing response in order to understand which operations are allowed
     if(!resp) return true; // Not authorized
     // Following should be somehow made easier
@@ -248,7 +181,7 @@ bool ARexJob::is_allowed(bool fast) {
       entry.NewChild("allow").NewChild("read");
     };
     ArcSec::Response *resp;
-    resp=eval->evaluate(request,policy);
+    resp=eval->evaluate(request,policy.Ptr());
     if(resp) {
       ArcSec::ResponseList& rlist = resp->getResponseItems();
       for(int n = 0; n<rlist.size(); ++n) {
@@ -262,7 +195,7 @@ bool ARexJob::is_allowed(bool fast) {
       entry["allow"].Destroy();
       entry.NewChild("allow").NewChild("write");
     };
-    resp=eval->evaluate(request,policy);
+    resp=eval->evaluate(request,policy.Ptr());
     if(resp) {
       ArcSec::ResponseList& rlist = resp->getResponseItems();
       for(int n = 0; n<rlist.size(); ++n) {
@@ -283,7 +216,7 @@ ARexJob::ARexJob(const std::string& id,ARexGMConfig& config,Arc::Logger& logger,
   if(id_.empty()) return;
   if(!config_) { id_.clear(); return; };
   // Reading essential information about job
-  if(!job_local_read_file(id_,*config_.User(),job_)) { id_.clear(); return; };
+  if(!job_local_read_file(id_,config_.GmConfig(),job_)) { id_.clear(); return; };
   // Checking if user is allowed to do anything with that job
   if(!is_allowed(fast_auth_check)) { id_.clear(); return; };
   if(!(allowed_to_see_ || allowed_to_maintain_)) { id_.clear(); return; };
@@ -291,7 +224,7 @@ ARexJob::ARexJob(const std::string& id,ARexGMConfig& config,Arc::Logger& logger,
 
 ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& credentials,const std::string& clientid, Arc::Logger& logger, JobIDGenerator& idgenerator,  Arc::XMLNode migration):id_(""),logger_(logger),config_(config) {
   if(!config_) return;
-  DelegationStores* deleg = config_.User()->Env().delegations();
+  DelegationStores* deleg = config_.GmConfig().Delegations();
   // New job is created here
   // First get and acquire new id
   if(!make_job_id()) return;
@@ -303,18 +236,29 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
     jsdl.New(jsdldoc);
     jsdldoc.GetDoc(job_desc_str);
   };
-  // Store description
-  std::string fname = config_.User()->ControlDir() + "/job." + id_ + ".description";
-  if(!job_description_write_file(fname,job_desc_str)) {
+  // Choose session directory
+  std::string sessiondir;
+  if (!ChooseSessionDir(id_, sessiondir)) {
     delete_job_id();
-    failure_="Failed to store job RSL description";
+    failure_="Failed to find valid session directory";
     failure_type_=ARexJobInternalError;
     return;
   };
-  // Analyze job descrption (checking, substituting, etc)
-  std::string acl("");
+  job_.sessiondir = sessiondir+"/"+id_;
+  GMJob job(id_,Arc::User(config_.User().get_uid()),job_.sessiondir,JOB_STATE_ACCEPTED);
+  // Store description
+  if(!job_description_write_file(job,config_.GmConfig(),job_desc_str)) {
+    delete_job_id();
+    failure_="Failed to store job description";
+    failure_type_=ARexJobInternalError;
+    return;
+  };
+  // Analyze job description (checking, substituting, etc)
+  JobDescriptionHandler job_desc_handler(config.GmConfig());
   Arc::JobDescription desc;
-  if((failure_type_=setfail(parse_job_req(fname.c_str(),job_,desc,&acl,&failure_))) != ARexJobNoError) {
+  JobReqResult parse_result = job_desc_handler.parse_job_req(id_,job_,desc,true);
+  if((failure_type_=setfail(parse_result)) != ARexJobNoError) {
+    failure_ = parse_result.failure;
     if(failure_.empty()) {
       failure_="Failed to parse job description";
       failure_type_=ARexJobInternalError;
@@ -322,6 +266,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
     delete_job_id();
     return;
   };
+  std::string acl(parse_result.acl);
   if((!job_.action.empty()) && (job_.action != "request")) {
     failure_="Wrong action in job request: "+job_.action;
     failure_type_=ARexJobInternalError;
@@ -331,8 +276,8 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   // Check for proper LRMS name in request. If there is no LRMS name
   // in user configuration that means service is opaque frontend and
   // accepts any LRMS in request.
-  if((!job_.lrms.empty()) && (!config_.User()->DefaultLRMS().empty())) {
-    if(job_.lrms != config_.User()->DefaultLRMS()) {
+  if((!job_.lrms.empty()) && (!config_.GmConfig().DefaultLRMS().empty())) {
+    if(job_.lrms != config_.GmConfig().DefaultLRMS()) {
       failure_="Requested LRMS is not supported by this service";
       failure_type_=ARexJobInternalError;
       //failure_type_=ARexJobDescriptionLogicalError;
@@ -340,18 +285,18 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
       return;
     };
   };
-  if(job_.lrms.empty()) job_.lrms=config_.User()->DefaultLRMS();
+  if(job_.lrms.empty()) job_.lrms=config_.GmConfig().DefaultLRMS();
   // Check for proper queue in request.
-  if(job_.queue.empty()) job_.queue=config_.User()->DefaultQueue();
+  if(job_.queue.empty()) job_.queue=config_.GmConfig().DefaultQueue();
   if(job_.queue.empty()) {
     failure_="Request has no queue defined";
     failure_type_=ARexJobDescriptionMissingError;
     delete_job_id();
     return;
   };
-  if(config_.Queues().size() > 0) { // If no queues configured - service takes any
-    for(std::list<std::string>::const_iterator q = config_.Queues().begin();;++q) {
-      if(q == config_.Queues().end()) {
+  if(!config_.GmConfig().Queues().empty()) { // If no queues configured - service takes any
+    for(std::list<std::string>::const_iterator q = config_.GmConfig().Queues().begin();;++q) {
+      if(q == config_.GmConfig().Queues().end()) {
         failure_="Requested queue "+job_.queue+" does not match any of available queues";
         //failure_type_=ARexJobDescriptionLogicalError;
         failure_type_=ARexJobInternalError;
@@ -412,30 +357,12 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   // TODO: TotalWallTime;
   // TODO: NodeAccess;
   // TODO: CEType;
-  for(std::list<Arc::Software>::const_iterator rte =
-                    desc.Resources.RunTimeEnvironment.getSoftwareList().begin();
-                    rte != desc.Resources.RunTimeEnvironment.getSoftwareList().end();++rte) {
-    if(!rte->getOptions().empty()) {
-      failure_="Options in RTE not supported yet";
-      failure_type_=ARexJobDescriptionUnsupportedError;
-      delete_job_id();
-      return;
-    };
-  };
-  // Check for NumberOfSlots is removed because in current implementation
-  // it is used to really represent number of requested cores/cpus.
-  // This must be adjusted later when full support for slots is ready.
-  if((desc.Resources.SlotRequirement.ExclusiveExecution != Arc::SlotRequirementType::EE_DEFAULT) ||
-     (desc.Resources.SlotRequirement.SlotsPerHost > 1) /* ||
-     (desc.Resources.SlotRequirement.NumberOfSlots > 1)*/) {
-    failure_="SlotRequirement is not fully supported yet";
-    failure_type_=ARexJobDescriptionUnsupportedError;
-    delete_job_id();
-    return;
-  };
-  if((!desc.Resources.ParallelEnvironment.Type.empty()) ||
-     (!desc.Resources.ParallelEnvironment.Version.empty())) {
-    failure_="ParallelEnvironment is not supported yet. Use RunTimeEnvironment instead";
+  // Check that the SlotRequirements make sense.
+  // I.e. that SlotsPerHost do not exceed total Slots
+  // and that SlotsPerHost is a divisor of total Slots
+  if((desc.Resources.SlotRequirement.SlotsPerHost > desc.Resources.SlotRequirement.NumberOfSlots) ||
+     (desc.Resources.SlotRequirement.NumberOfSlots % desc.Resources.SlotRequirement.SlotsPerHost != 0)) {
+    failure_="SlotsPerHost exceeding NumberOfSlots is not supported";
     failure_type_=ARexJobDescriptionUnsupportedError;
     delete_job_id();
     return;
@@ -467,11 +394,10 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
                                                    f != desc.DataStaging.OutputFiles.end();++f) {
         for(std::list<Arc::TargetType>::iterator t = f->Targets.begin();t != f->Targets.end();++t) {
           if(!(t->DelegationID.empty())) {
-            std::string path = (*deleg)[config_.User()->DelegationDir()].
+            std::string path = (*deleg)[config_.GmConfig().DelegationDir()].
                                       FindCred(t->DelegationID,config_.GridName());
             if(!path.empty()) {
-              std::ifstream fcred(path.c_str());
-              std::getline(fcred,cred,'\0');
+              Arc::FileRead(path, cred);
               if(!cred.empty()) {
                 need_delegation = false; 
                 break;
@@ -486,11 +412,10 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
                                                   f != desc.DataStaging.InputFiles.end();++f) {
         for(std::list<Arc::SourceType>::iterator t = f->Sources.begin();t != f->Sources.end();++t) {
           if(!(t->DelegationID.empty())) {
-            std::string path = (*deleg)[config_.User()->DelegationDir()].
+            std::string path = (*deleg)[config_.GmConfig().DelegationDir()].
                                       FindCred(t->DelegationID,config_.GridName());
             if(!path.empty()) {
-              std::ifstream fcred(path.c_str());
-              std::getline(fcred,cred,'\0');
+              Arc::FileRead(path, cred);
               if(!cred.empty()) {
                 need_delegation = false;
                 break;
@@ -550,19 +475,9 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
       };
     };
   };
-  // Choose session directory
-  std::string sessiondir;
-  if (!ChooseSessionDir(id_, sessiondir)) {
-    delete_job_id();
-    failure_="Failed to find valid session directory";
-    failure_type_=ARexJobInternalError;
-    return;
-  };
-  config_.User()->SetSessionRoot(sessiondir);
   // Write local file
-  JobDescription job(id_,config_.User()->SessionRoot()+"/"+id_,JOB_STATE_ACCEPTED);
   job.set_local(&job_); // need this for write_grami
-  if(!job_local_write_file(job,*config_.User(),job_)) {
+  if(!job_local_write_file(job,config_.GmConfig(),job_)) {
     delete_job_id();
     failure_="Failed to create job description";
     failure_type_=ARexJobInternalError;
@@ -581,7 +496,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
     };
   };
   // Write grami file
-  if(!write_grami(desc,job,*config_.User(),NULL)) {
+  if(!job_desc_handler.write_grami(desc,job,NULL)) {
     delete_job_id();
     failure_="Failed to create grami file";
     failure_type_=ARexJobInternalError;
@@ -589,7 +504,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   };
   // Write ACL file
   if(!acl.empty()) {
-    if(!job_acl_write_file(id_,*config.User(),acl)) {
+    if(!job_acl_write_file(id_,config.GmConfig(),acl)) {
       delete_job_id();
       failure_="Failed to process/store job ACL";
       failure_type_=ARexJobInternalError;
@@ -600,7 +515,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   {
     // talk to external plugin to ask if we can proceed
     std::list<ContinuationPlugins::result_t> results;
-    config_.Plugins().run(job,*config_.User(),results);
+    config_.GmConfig().ContPlugins()->run(job,config_.GmConfig(),results);
     std::list<ContinuationPlugins::result_t>::iterator result = results.begin();
     while(result != results.end()) {
       // analyze results
@@ -651,7 +566,7 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   };
 */
   // Create session directory
-  if(!job_session_create(job,*config_.User())) {
+  if(!config_.GmConfig().CreateSessionDirectory(job.SessionDir(), job.get_user())) {
     delete_job_id();
     failure_="Failed to create session directory";
     failure_type_=ARexJobInternalError;
@@ -659,25 +574,25 @@ ARexJob::ARexJob(Arc::XMLNode jsdl,ARexGMConfig& config,const std::string& crede
   };
   // Create input status file to tell downloader we
   // are handling input in clever way.
-  job_input_status_add_file(job,*config_.User());
+  job_input_status_add_file(job,config_.GmConfig());
   // Create status file (do it last so GM picks job up here)
-  if(!job_state_write_file(job,*config_.User(),JOB_STATE_ACCEPTED)) {
+  if(!job_state_write_file(job,config_.GmConfig(),JOB_STATE_ACCEPTED)) {
     delete_job_id();
     failure_="Failed registering job in grid-manager";
     failure_type_=ARexJobInternalError;
     return;
   };
   // Put lock on delegated credentials
-  if(deleg) (*deleg)[config_.User()->DelegationDir()].LockCred(id_,deleg_ids,config_.GridName());
+  if(deleg) (*deleg)[config_.GmConfig().DelegationDir()].LockCred(id_,deleg_ids,config_.GridName());
 
-  SignalFIFO(*config_.User());
+  SignalFIFO(config_.GmConfig().ControlDir());
   return;
 }
 
 bool ARexJob::GetDescription(Arc::XMLNode& jsdl) {
   if(id_.empty()) return false;
   std::string sdesc;
-  if(!job_description_read_file(id_,*config_.User(),sdesc)) return false;
+  if(!job_description_read_file(id_,config_.GmConfig(),sdesc)) return false;
   Arc::XMLNode xdesc(sdesc);
   if(!xdesc) return false;
   jsdl.Replace(xdesc);
@@ -686,15 +601,15 @@ bool ARexJob::GetDescription(Arc::XMLNode& jsdl) {
 
 bool ARexJob::Cancel(void) {
   if(id_.empty()) return false;
-  JobDescription job_desc(id_,"");
-  if(!job_cancel_mark_put(job_desc,*config_.User())) return false;
+  GMJob job(id_,Arc::User(config_.User().get_uid()));
+  if(!job_cancel_mark_put(job,config_.GmConfig())) return false;
   return true;
 }
 
 bool ARexJob::Clean(void) {
   if(id_.empty()) return false;
-  JobDescription job_desc(id_,"");
-  if(!job_clean_mark_put(job_desc,*config_.User())) return false;
+  GMJob job(id_,Arc::User(config_.User().get_uid()));
+  if(!job_clean_mark_put(job,config_.GmConfig())) return false;
   return true;
 }
 
@@ -708,7 +623,7 @@ bool ARexJob::Resume(void) {
     // Job run out of number of allowed retries.
     return false;
   };
-  if(!job_restart_mark_put(JobDescription(id_,""),*config_.User())) {
+  if(!job_restart_mark_put(GMJob(id_,Arc::User(config_.User().get_uid())),config_.GmConfig())) {
     // Failed to report restart request.
     return false;
   };
@@ -722,30 +637,30 @@ std::string ARexJob::State(void) {
 
 std::string ARexJob::State(bool& job_pending) {
   if(id_.empty()) return "";
-  job_state_t state = job_state_read_file(id_,*config_.User(),job_pending);
+  job_state_t state = job_state_read_file(id_,config_.GmConfig(),job_pending);
   if(state > JOB_STATE_UNDEFINED) state=JOB_STATE_UNDEFINED;
   return states_all[state].name;
 }
 
 bool ARexJob::Failed(void) {
   if(id_.empty()) return false;
-  return job_failed_mark_check(id_,*config_.User());
+  return job_failed_mark_check(id_,config_.GmConfig());
 }
 
 std::string ARexJob::FailedState(std::string& cause) {
   std::string state;
-  job_local_read_failed(id_,*config_.User(),state,cause);
+  job_local_read_failed(id_,config_.GmConfig(),state,cause);
   return state;
 }
 
 Arc::Time ARexJob::Created(void) {
-  time_t t = job_description_time(id_,*config_.User());
+  time_t t = job_description_time(id_,config_.GmConfig());
   if(t == 0) return Arc::Time(); // ???
   return Arc::Time(t);
 }
 
 Arc::Time ARexJob::Modified(void) {
-  time_t t = job_state_time(id_,*config_.User());
+  time_t t = job_state_time(id_,config_.GmConfig());
   if(t == 0) return Arc::Time(); // ???
   return Arc::Time(t);
 }
@@ -753,18 +668,18 @@ Arc::Time ARexJob::Modified(void) {
 bool ARexJob::UpdateCredentials(const std::string& credentials) {
   if(id_.empty()) return false;
   if(!update_credentials(credentials)) return false;
-  JobDescription job(id_,config_.User()->SessionRoot(id_)+"/"+id_,JOB_STATE_ACCEPTED);
-  if(!job_local_write_file(job,*config_.User(),job_)) return false;
+  GMJob job(id_,Arc::User(config_.User().get_uid()),config_.GmConfig().SessionRoot(id_)+"/"+id_,JOB_STATE_ACCEPTED);
+  if(!job_local_write_file(job,config_.GmConfig(),job_)) return false;
   return true;
 }
 
 bool ARexJob::update_credentials(const std::string& credentials) {
   if(credentials.empty()) return true;
-  std::string fname=config_.User()->ControlDir()+"/job."+id_+".proxy";
+  std::string fname=config_.GmConfig().ControlDir()+"/job."+id_+".proxy";
   ::unlink(fname.c_str());
   int h=::open(fname.c_str(),O_WRONLY | O_CREAT | O_EXCL,0600);
   if(h == -1) return false;
-  fix_file_owner(fname,*config_.User());
+  fix_file_owner(fname,config_.User());
   const char* s = credentials.c_str();
   int ll = credentials.length();
   int l = 0;
@@ -807,7 +722,7 @@ bool ARexJob::make_job_id(void) {
     //    Arc::tostring((unsigned int)time(NULL))+
     //    Arc::tostring(rand(),1);
     Arc::GUID(id_);
-    std::string fname=config_.User()->ControlDir()+"/job."+id_+".description";
+    std::string fname=config_.GmConfig().ControlDir()+"/job."+id_+".description";
     struct stat st;
     if(stat(fname.c_str(),&st) == 0) continue;
     int h = ::open(fname.c_str(),O_RDWR | O_CREAT | O_EXCL,0600);
@@ -816,15 +731,15 @@ bool ARexJob::make_job_id(void) {
     int err = errno;
     if(h == -1) {
       if(err == EEXIST) continue;
-      logger_.msg(Arc::ERROR, "Failed to create file in %s", config_.User()->ControlDir());
+      logger_.msg(Arc::ERROR, "Failed to create file in %s", config_.GmConfig().ControlDir());
       id_="";
       return false;
     };
-    fix_file_owner(fname,*config_.User());
+    fix_file_owner(fname,config_.User());
     close(h);
     return true;
   };
-  logger_.msg(Arc::ERROR, "Out of tries while allocating new job ID in %s", config_.User()->ControlDir());
+  logger_.msg(Arc::ERROR, "Out of tries while allocating new job ID in %s", config_.GmConfig().ControlDir());
   id_="";
   return false;
 }
@@ -832,8 +747,8 @@ bool ARexJob::make_job_id(void) {
 bool ARexJob::delete_job_id(void) {
   if(!config_) return true;
   if(!id_.empty()) {
-    job_clean_final(JobDescription(id_,
-                config_.User()->SessionRoot(id_)+"/"+id_),*config_.User());
+    job_clean_final(GMJob(id_,Arc::User(config_.User().get_uid()),
+                config_.GmConfig().SessionRoot(id_)+"/"+id_),config_.GmConfig());
     id_="";
   };
   return true;
@@ -841,7 +756,7 @@ bool ARexJob::delete_job_id(void) {
 
 int ARexJob::TotalJobs(ARexGMConfig& config,Arc::Logger& /* logger */) {
   ContinuationPlugins plugins;
-  JobsList jobs(*config.User(),plugins);
+  JobsList jobs(config.GmConfig());
   jobs.ScanAllJobs();
   return jobs.size();
 }
@@ -849,7 +764,7 @@ int ARexJob::TotalJobs(ARexGMConfig& config,Arc::Logger& /* logger */) {
 std::list<std::string> ARexJob::Jobs(ARexGMConfig& config,Arc::Logger& logger) {
   std::list<std::string> jlist;
   ContinuationPlugins plugins;
-  JobsList jobs(*config.User(),plugins);
+  JobsList jobs(config.GmConfig());
   jobs.ScanAllJobs();
   JobsList::iterator i = jobs.begin();
   for(;i!=jobs.end();++i) {
@@ -861,7 +776,7 @@ std::list<std::string> ARexJob::Jobs(ARexGMConfig& config,Arc::Logger& logger) {
 
 std::string ARexJob::SessionDir(void) {
   if(id_.empty()) return "";
-  return config_.User()->SessionRoot(id_)+"/"+id_;
+  return config_.GmConfig().SessionRoot(id_)+"/"+id_;
 }
 
 std::string ARexJob::LogDir(void) {
@@ -901,35 +816,35 @@ Arc::FileAccess* ARexJob::CreateFile(const std::string& filename) {
     return NULL;
   };
   int lname = fname.length();
-  fname = config_.User()->SessionRoot(id_)+"/"+id_+"/"+fname;
+  fname = config_.GmConfig().SessionRoot(id_)+"/"+id_+"/"+fname;
   // First try to create/open file
-  Arc::FileAccess* fa = new Arc::FileAccess();
+  Arc::FileAccess* fa = Arc::FileAccess::Acquire();
   if(!*fa) {
     delete fa;
     return NULL;
   };
-  if(!fa->setuid(config_.User()->get_uid(),config_.User()->get_gid())) {
-    delete fa;
+  if(!fa->fa_setuid(config_.User().get_uid(),config_.User().get_gid())) {
+    Arc::FileAccess::Release(fa);
     return NULL;
   };
-  if(!fa->open(fname,O_WRONLY | O_CREAT,S_IRUSR | S_IWUSR)) {
+  if(!fa->fa_open(fname,O_WRONLY | O_CREAT,S_IRUSR | S_IWUSR)) {
     if(fa->geterrno() != ENOENT) {
-      delete fa;
+      Arc::FileAccess::Release(fa);
       return NULL;
     };
     std::string::size_type n = fname.rfind('/');
     if((n == std::string::npos) || (n < (fname.length()-lname))) {
-      delete fa;
+      Arc::FileAccess::Release(fa);
       return NULL;
     };
-    if(!fa->mkdirp(fname.substr(0,n),S_IRUSR | S_IWUSR | S_IXUSR)) {
+    if(!fa->fa_mkdirp(fname.substr(0,n),S_IRUSR | S_IWUSR | S_IXUSR)) {
       if(fa->geterrno() != EEXIST) {
-        delete fa;
+        Arc::FileAccess::Release(fa);
         return NULL;
       };
     };
-    if(!fa->open(fname,O_WRONLY | O_CREAT,S_IRUSR | S_IWUSR)) {
-      delete fa;
+    if(!fa->fa_open(fname,O_WRONLY | O_CREAT,S_IRUSR | S_IWUSR)) {
+      Arc::FileAccess::Release(fa);
       return NULL;
     };
   };
@@ -944,23 +859,23 @@ Arc::FileAccess* ARexJob::OpenFile(const std::string& filename,bool for_read,boo
     failure_type_=ARexJobInternalError;
     return NULL;
   };
-  fname = config_.User()->SessionRoot(id_)+"/"+id_+"/"+fname;
+  fname = config_.GmConfig().SessionRoot(id_)+"/"+id_+"/"+fname;
   int flags = 0;
   if(for_read && for_write) { flags=O_RDWR; }
   else if(for_read) { flags=O_RDONLY; }
   else if(for_write) { flags=O_WRONLY; }
-  //return Arc::FileOpen(fname,flags,config_.User()->get_uid(),config_.User()->get_gid(),0);
-  Arc::FileAccess* fa = new Arc::FileAccess();
+  //return Arc::FileOpen(fname,flags,config_.User().get_uid(),config_.User().get_gid(),0);
+  Arc::FileAccess* fa = Arc::FileAccess::Acquire();
   if(*fa) {
-    if(fa->setuid(config_.User()->get_uid(),config_.User()->get_gid())) {
-      if(fa->open(fname,flags,0)) {
+    if(fa->fa_setuid(config_.User().get_uid(),config_.User().get_gid())) {
+      if(fa->fa_open(fname,flags,0)) {
         return fa;
       };
     };
   };
   failure_="Failed opening file - "+Arc::StrError(fa->geterrno());
   failure_type_=ARexJobInternalError;
-  delete fa;
+  Arc::FileAccess::Release(fa);
   return NULL;
 }
 
@@ -973,34 +888,34 @@ Arc::FileAccess* ARexJob::OpenDir(const std::string& dirname) {
     return NULL;
   };
   //if(dname.empty()) return NULL;
-  dname = config_.User()->SessionRoot(id_)+"/"+id_+"/"+dname;
-  Arc::FileAccess* fa = new Arc::FileAccess();
+  dname = config_.GmConfig().SessionRoot(id_)+"/"+id_+"/"+dname;
+  Arc::FileAccess* fa = Arc::FileAccess::Acquire();
   if(*fa) {
-    if(fa->setuid(config_.User()->get_uid(),config_.User()->get_gid())) {
-      if(fa->opendir(dname)) {
+    if(fa->fa_setuid(config_.User().get_uid(),config_.User().get_gid())) {
+      if(fa->fa_opendir(dname)) {
         return fa;
       };
     };
   };
   failure_="Failed opening directory - "+Arc::StrError(fa->geterrno());
   failure_type_=ARexJobInternalError;
-  delete fa;
+  Arc::FileAccess::Release(fa);
   return NULL;
 }
 
 int ARexJob::OpenLogFile(const std::string& name) {
   if(id_.empty()) return -1;
   if(strchr(name.c_str(),'/')) return -1;
-  std::string fname = config_.User()->ControlDir() + "/job." + id_ + "." + name;
+  std::string fname = config_.GmConfig().ControlDir() + "/job." + id_ + "." + name;
   return ::open(fname.c_str(),O_RDONLY);
 }
 
 std::list<std::string> ARexJob::LogFiles(void) {
   std::list<std::string> logs;
   if(id_.empty()) return logs;
-  std::string dname = config_.User()->ControlDir();
+  std::string dname = config_.GmConfig().ControlDir();
   std::string prefix = "job." + id_ + ".";
-  // TODO: scanning is performace bottleneck. Use matching instead.
+  // TODO: scanning is performance bottleneck. Use matching instead.
   Glib::Dir* dir = new Glib::Dir(dname);
   if(!dir) return logs;
   for(;;) {
@@ -1017,25 +932,25 @@ std::string ARexJob::GetFilePath(const std::string& filename) {
   if(id_.empty()) return "";
   std::string fname = filename;
   if(!normalize_filename(fname)) return "";
-  if(fname.empty()) config_.User()->SessionRoot(id_)+"/"+id_;
-  return config_.User()->SessionRoot(id_)+"/"+id_+"/"+fname;
+  if(fname.empty()) config_.GmConfig().SessionRoot(id_)+"/"+id_;
+  return config_.GmConfig().SessionRoot(id_)+"/"+id_+"/"+fname;
 }
 
 bool ARexJob::ReportFileComplete(const std::string& filename) {
   if(id_.empty()) return "";
   std::string fname = filename;
   if(!normalize_filename(fname)) return false;
-  return job_input_status_add_file(JobDescription(id_,""),*config_.User(),"/"+fname);
+  return job_input_status_add_file(GMJob(id_,Arc::User(config_.User().get_uid())),config_.GmConfig(),"/"+fname);
 }
 
 bool ARexJob::ReportFilesComplete(void) {
   if(id_.empty()) return "";
-  return job_input_status_add_file(JobDescription(id_,""),*config_.User(),"/");
+  return job_input_status_add_file(GMJob(id_,Arc::User(config_.User().get_uid())),config_.GmConfig(),"/");
 }
 
 std::string ARexJob::GetLogFilePath(const std::string& name) {
   if(id_.empty()) return "";
-  return config_.User()->ControlDir() + "/job." + id_ + "." + name;
+  return config_.GmConfig().ControlDir() + "/job." + id_ + "." + name;
 }
 
 bool ARexJob::ChooseSessionDir(const std::string& /* jobid */, std::string& sessiondir) {
