@@ -10,11 +10,13 @@
 #include <arc/Utils.h>
 #include <arc/communication/ClientInterface.h>
 #include <sstream>
+#include <string> 
 
 namespace Arc
 {
   ApelDestination::ApelDestination(JobLogFile& joblog):
     logger(Arc::Logger::rootLogger, "JURA.ApelDestination"),
+    use_ssl("false"),
     urn(0),
     sequence(0),
     usagerecordset(Arc::NS("","http://eu-emi.eu/namespaces/2012/11/computerecord"),
@@ -69,6 +71,9 @@ namespace Arc
         endpoint=url.Path();
       }
 
+    //read the previous aggregation records
+    aggregationManager = new CARAggregation(host,port,topic, true);
+
     //Get Batch Size:
     //Default value:
     max_ur_set_size=JURA_DEFAULT_MAX_APEL_UR_SET_SIZE;
@@ -79,7 +84,12 @@ namespace Arc
          std::istringstream is(urbatch);
         is>>max_ur_set_size;
       }
-
+    std::string o_use_ssl=joblog["jobreport_option_use_ssl"];
+    std::transform(o_use_ssl.begin(), o_use_ssl.end(), o_use_ssl.begin(), tolower);
+    if (o_use_ssl == "true")
+      {
+          use_ssl="true";
+      }
   }
 
   void ApelDestination::report(Arc::JobLogFile &joblog)
@@ -95,6 +105,7 @@ namespace Arc
           {
             usagerecordset.NewChild(usagerecord);
             ++urn;
+            aggregationManager->UpdateAggregationRecord(usagerecord);
           }
         else
           {
@@ -114,6 +125,7 @@ namespace Arc
     if (urn>0)
       // Send the remaining URs and delete job log files.
       submit_batch();
+    delete aggregationManager;
   }
 
   int ApelDestination::submit_batch()
@@ -133,6 +145,11 @@ namespace Arc
 
     if (status.isOk())
       {
+        // Save the modified aggregation records
+        aggregationManager->save_records();
+        // Reported the new synch record
+        aggregationManager->Reporting_records();
+
         // Delete log files
         for (std::list<JobLogFile>::iterator jp=joblogs.begin();
              jp!=joblogs.end();
@@ -218,7 +235,7 @@ namespace Arc
     }
 
     //Save all records into the default folder.
-    std::string default_path = "/var/spool/arc/ssm/";
+    std::string default_path = (std::string)JURA_DEFAULT_DIR_PREFIX + "/ssm/";
     struct stat st;
     //directory check
     if (stat(default_path.c_str(), &st) != 0) {
@@ -259,7 +276,7 @@ namespace Arc
                );
     }
     int retval;
-    //ssmsend <hostname> <port> <topic> <key path> <cert path> <cadir path> <messages path>"
+    //ssmsend <hostname> <port> <topic> <key path> <cert path> <cadir path> <messages path> <use_ssl>"
     std::string command;
     std::vector<std::string> ssm_pathes;
     std::string exec_cmd = "ssmsend";
@@ -298,6 +315,7 @@ namespace Arc
     command += " " + cfg.cert;   //certificate
     command += " " + cfg.cadir;  //cadir
     command += " " + default_path; //messages path
+    command += " " + use_ssl;    //use_ssl
     command += "";
 
     retval = system(command.c_str());
